package physbeans.func;

import physbeans.math.*;
import static java.lang.Math.*;

/**
 * Function that computes the (generalized) eigenvectors for given mass
 * matrix M and stiffness matrix C.
 * <p>On input of the index i it returns the corrsponding eigenvector
 * x<sub>i</sub>.
 */
public class EigenVectorFunction extends GenericVectorFunction {
  
  // possible types of interpolators
  public static final int NORM_VECTOR = 0;
  public static final int NORM_FIRST_ELEMENT = 1;
  public static final int NORM_LARGEST_ELEMENT = 2;
  
  protected int index;
  protected int normType;
  protected GeneralizedEigenvalueDecomposition eig;
  
  /** Creates a new instance of EigenVectorFunction */
  public EigenVectorFunction() {
    super(0, 1, 3);
    index = 0;
    normType = NORM_VECTOR;
  }
  
  /**
   * Computes the eigenvector with given index.
   */
  protected void compute() {
    if (eig == null) {
      return;
    }
    
    Matrix Phi = eig.getPhi();
    DVector v = Phi.getColumnVector(index);
    normalizeVector(v);
    outputValues[0] = v;
    
  }
  
  /**
   * Gets index of eigenvector to compute.
   * <p>The first eigenvector has index 0!
   */
  public int getIndex() {
    return index;
  }
  
  /**
   * Gets index of eigenvector to compute.
   * <p>The first eigenvector has index 0!
   */
  public void setIndex(int i) {
    index = i;
    if (autoTriggered) {
      trigger();
    }
  }
  
  /**
   * Defines the matrices M and C defining the generalized eigenvalue problem.
   */
  public void setMatrices(Matrix C, Matrix M) {
    eig = new GeneralizedEigenvalueDecomposition(C, M);
    trigger();
  }
  
  /**
   * Gets norm type of eigenvectors.
   * Possible values: vector norm, first element, largest element.
   */
  public int getNormType() {
    return normType;
  }
  
  /**
   * Sets norm type of eigenvectors.
   * Possible values: vector norm, first element, largest element.
   */
  public void setNormType(int t) {
    normType = t;
    trigger();
  }
  
  /**
   * Scales vector v according to norm type.
   */
  protected void normalizeVector(DVector v) {
    double s = 1.0;
    switch (normType) {
      case NORM_VECTOR:
        s = v.norm();
        break;
      case NORM_FIRST_ELEMENT:
        s = v.get(0);
        break;
      case NORM_LARGEST_ELEMENT:
        s = abs(v.max()) > abs(v.min())? v.max() : v.min(); 
        break;
    }
    v.timesEquals(1.0/s);
  }
}
