package physbeans.func;

import physbeans.event.*;
import physbeans.math.*;

/**
 * Describes a function of n (vector) input values with m (vector) output values.
 * <p>Output vectors can have different lengths, set in advance. Alternatively one
 * can set the length of an output vector to a negative value -n. In this case
 * the length of input vector (n-1) is used.
 * <p>If in autotrigger mode, events are either sent only after new values for
 * all input vectors have arrived (combine mode) or each time a new input for
 * any input vector comes in.
 */
public abstract class GenericVectorFunction extends GenericFunction {
  
  // Input vectors are only references, output vectors are stored internally.
  
  protected DVector[] inputValues;
  protected DVector[] outputValues;
  protected int[] outputLength;
  protected boolean combineMode;
  
  protected StateCombiner states;
  
  /**
   * default constructor: 2 input vectors, one output vector of length 100
   */
  public GenericVectorFunction() {
    this(2, 1, 100);
  }
  
  /**
   * Constructs GenericVectorFunction with nIn input vectors and
   * nOut output vectors of identical length len
   */
  public GenericVectorFunction(int nIn, int nOut, int len) {
    inputValues = new DVector[nIn];
    outputValues = new DVector[nOut];
    outputLength = new int[nOut];
    
    for (int i = 0; i < nOut; i++) {
      setOutputLength(i, len);
    }
    
    autoTriggered = true;
    combineMode = false;
    states = new StateCombiner(nIn);
  }
  
  /**
   * Computes output vectors and informs listeners.
   */
  public void trigger() {
    // check whether all input values are set already
    for (int i = 0; i < inputValues.length; i++) {
      if  (inputValues[i] == null) return;
    }
    
    // create dynamically sized output vectors
    for (int i = 0; i < outputValues.length; i++) {
      if  (outputLength[i] < 0) {
        int newSize = inputValues[-outputLength[i]-1].getDimension();
        outputValues[i] = new DVector(newSize);
      }
    }
    
    compute();  // compute new output values
    fireVectorEvent(new VectorEvent(this, outputValues)); // inform listeners
  }
  
  /**
   * Gets the number of vector inputs.
   */
  public int getNInput() {
    return inputValues.length;
  }
  
  /**
   * Sets the number of input values.
   */
  public void setNInput(int nIn) {
    inputValues = new DVector[nIn];
    states = new StateCombiner(nIn);
  }
  
  /**
   * Gets the number of vector outputs.
   */
  public int getNOutput() {
    return outputValues.length;
  }
  
  /**
   * Sets the number of vector outputs.
   */
  public void setNOutput(int nOut) {
    outputValues = new DVector[nOut];
    outputLength = new int[nOut];
  }
  
  /**
   * Gets length of output vector i
   */
  public int getOutputLength(int i) {
    return outputLength[i];
  }
  
  /**
   * Gets lengths of all output vectors.
   */
  public int[] getOutputLength() {
    return outputLength;
  }
  
  /**
   * Sets length of output vector i
   */
  public void setOutputLength(int i, int len) {
    outputLength[i] = len;
    if (len >= 0) {
      outputValues[i] = new DVector(len);
    }
    // negative length -> use input vector length dynamically
  }
  
  /**
   * Sets lengths of all output vectors.
   */
  public void setOutputLength(int[] len) {
    for (int i = 0; i < outputValues.length; i++) {
      setOutputLength(i, len[i]);
    }
  }
  
  /**
   * Sets the number of output values for all output vectors to len.
   */
  public void setAllOutputLengths(int len) {
    for (int i = 0; i < outputValues.length; i++) {
      setOutputLength(i, len);
    }
  }
  
  /**
   * Sets input vector n with values v.
   * <p>Triggers the function if  autoTriggered is active and
   * either combineMode is false or new inputs have arrived for all vectors.
   * After one trigger event in combine mode all inputs have to be set again.
   */
  public void setInputVector(int n, DVector v) {
    inputValues[n] = v;
    states.setState(n);
    
    if (autoTriggered && (!combineMode || states.getCombinedState())) {
      trigger();
      if (combineMode) {
        states.reset();
      }
    }
  }
  
  /**
   * Sets all input vectors.<br>
   * Triggers the function if autoTriggered.
   */
  public void setAllInputVectors(DVector[] vecs) {
    inputValues = vecs;
    
    if (autoTriggered) {
      trigger();
    }
  }
  
  /**
   * Gets the length of the first output vector.
   * <P>Conveniance function for the case nOut = 1.
   */
  public int getOutputSize() {
    return getOutputLength(0);
  }
  
  /**
   * Sets the length of the first output vector.
   * <P>Conveniance function for the case nOut = 1.
   */
  public void setOutputSize(int len) {
    setOutputLength(0, len);
  }
  
  /**
   * Gets the first output vector.
   * <P>Conveniance function for the case nOut = 1.
   */
  public DVector getOutputVector() {
    return outputValues[0];
  }
  
  /**
   * Sets values of first input Vector.
   * Conveniance function for the case nIn = 1.
   */
  public void setInputVector(DVector v) {
    setInputVector(0, v);
  }
  
  /**
   * Gets first value of output vector.
   * <P>Conveniance function for the case nOut = 1.
   */
  public double getOutputValues(int i) {
    return getOutputVector().get(i);
  }
  
  /**
   * Gets the i'th output vector.
   */
  public DVector getOutputVector(int i) {
    return outputValues[i];
  }
  
  /**
   * Gets all output vectors.
   */
  public DVector[] getAllOutputVectors() {
    return outputValues;
  }
  
  /**
   * Gets the combine mode flag.
   * <p>If it is true (as well as autoTriggered) then the function triggers only
   * after all input vectors have been set. Otherwise each setting of an input
   * vector leads to a trigger event.
   * <p>After a trigger new values for all inputs have to arrive for the next
   * trigger.
   */
  public boolean isCombineMode() {
    return combineMode;
  }
  
  /**
   * Sets the combine mode flag.
   * <p>If it is true (as well as autoTriggered) then the function triggers only
   * after all input vectors have been set. Otherwise each setting of an input
   * vector leads to a trigger event.
   * <p>After a trigger new values for all inputs have to arrive for the next
   * trigger.
   */
  public void setCombineMode(boolean b) {
    combineMode = b;
  }
  
}
