package physbeans.func;

import physbeans.math.*;
import physbeans.model.*;
import java.io.*;

/**
 * Periodic function describing a gaussian pulse of constant area.
 * External parameter is the pulse width.
 * Additionally the offset of the function can be set to shift the function
 * in y direction.
 */
public class ImpulseFunction extends OneParameterFunction
implements Serializable, PeriodicFunctionWithParameter {
  
  protected double pulseWidth;
  protected double T;
  protected double A;
  
  /** Holds value of property offset. */
  protected double offset;
  
  
  /** Creates a new instance of ImpulseFunction */
  public ImpulseFunction() {
    super(-1);
    pulseWidth = 0.5;
    T = 2*Math.PI;
    A = 1.0;
    offset = 0.0;
  }
  
  /**
   * Gets the pulse width.
   * It is given as relative value, between 0 and 1.
   */
  public double getPulseWidth() {
    return pulseWidth;
  }
  
  /**
   * Sets the pulse width.
   * Input value is clamped to lie between 0 and 1.
   */
  public void setPulseWidth(double p) {
    if (p != 1.0) {
      p = p - Math.floor(p);
    }
    pulseWidth = p;
  }
  
  /**
   * Gets the period T.
   */
  public double getPeriod() {
    return T;
  }
  
  /**
   * Sets the period T.
   */
  public void setPeriod(double period) {
    T = period;
  }
  
  /**
   * Gets the parameter value.
   */
  public double getParameter() {
    return getPulseWidth();
  }
  
  /**
   * Sets the parameter
   */
  public void setParameter(double p) {
    setPulseWidth(p);
  }
  
  /**
   * Gets amplitude of the pulse.
   */
  public double getAmplitude() {
    return A;
  }
  
  /**
   * Sets amplitude of the pulse.
   */
  public void setAmplitude(double amplitude) {
    A = amplitude;
  }
  
  /**
   * Gets the offset.
   */
  public double getOffset() {
    return this.offset;
  }
  
  /**
   * Sets the offset.
   */
  public void setOffset(double offset) {
    this.offset = offset;
  }
  
  /*
   * Computes the function value for one given value t.
   */
  public double evaluate(double t) {
    // reduce t to basic intervall [0, T]
    t = t - T*Math.floor(t/T);
    
    // scale parameter value  [0, 1] -> [0, infinity]
    double width = -Math.log(1 - pulseWidth*pulseWidth);
    
    double expo = (t - T/2)/width;
    double x = A*Math.exp(-expo*expo)/width + offset;
    return x;
  }
  
}