package physbeans.func;

import physbeans.math.*;

/**
 * Describes a function of one parameter x = f(t).
 * The function can be applied to an input vector giving
 * a corresponding output vector through pointwise evaluation.
 */
public abstract class OneParameterFunction extends GenericVectorFunction {
  
  /**
   * Creates a function with one input and one output vector.<br>
   * The length of the output vector is dynamically adapted to the size of
   * the input vector.
   */
  public OneParameterFunction() {
    this(-1);
  }
  
  /**
   * Creates a function with one input and one output vector of given length.
   * <br>The length can be set to -1 in which case the size of the output vector 
   * is dynamically adapted to the size of the input vector.
   */
  public OneParameterFunction(int len) {
    super(1, 1, len);
  }
  
  /**
   * Computes all function values of the input vector.
   */
  protected void compute() {
    if (inputValues == null || inputValues[0] == null) {
      return;
    }
    for (int i = 0; i < inputValues[0].getDimension(); i++) {
      double res = evaluate(inputValues[0].get(i));
      outputValues[0].set(i,  res);
    }
  }
  
  /**
   * Gets first element of input vector.<br>
   * Conveniance function for the scalar case.
   */
  public double getOutputValue() {
    return getOutputVector().get(0);
  }
  
  /**
   * Sets input vector with one element.<br>
   * Conveniance function for the scalar case.
   */
  public void setInputValue(double t) {
    setInputVector(new DVector(t));
  }
  
  /*
   * Computes the function value for one given value t.
   */
  abstract public double evaluate(double t);
}
