package physbeans.inout;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.io.*;
import physbeans.views.*;

/**
 * Screen with several layers given by a set of paint functions (View's).
 * <P>Draws paint functions in order of decreasing depth (largest
 * depth == background etc.), provides a common world coordinate system and a
 * drawing area for all.
 * <p>
 * Views can be disabled individually. For this purpose they can be addressed
 * by an integer index, defined by the order of their depths. The first view
 * (number 0) ist the one with the largest depth value, i.e. the one at the
 * back of the stack.
 */
public class LayeredScreen extends Screen implements Serializable {
  
  protected boolean[] viewActive;
  protected Vector<View> views;
  
  /**
   * Creates default LayeredScreen with default world area and one view.
   */
  public LayeredScreen() {
    super();
    
    views = new Vector<View>();
    viewActive = new boolean[] {true};
  }
  
  /**
   * Sets parameters when the layout (basically the size) is fixed.
   */
  public void doLayout() {
    super.doLayout();
    
    // the attached views may have to adapt to the new size as well
    int n = views.size();
    for (int i = 0; i < n; i++) {
      if (viewActive[i % viewActive.length]) {
        views.get(i).doLayout();
      }
    }
  }
  
  /**
   * Paints layers using their views.
   */
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    
    Graphics2D g2 = (Graphics2D) g;
    AffineTransform oldTrafo = g2.getTransform();
    
    g2.setStroke(new BasicStroke(0.0f));  // as thin as possible
    g2.transform(trafo.getAffineTransform());
    
    int n = views.size();
    for (int i = 0; i < n; i++) {
      if (viewActive[i % viewActive.length]) {
        views.get(i).paintLayer(g2);
      }
    }
    g2.setTransform(oldTrafo);
    
  }
  
  /**
   * Adds a view to the list of views.
   */
  public void addView(View v) {
    if (!views.contains(v)) {
      views.add(v);
      v.setTrafo(trafo);
      v.setParent(this);
    }
    
    Collections.sort(views);  // sort in order of decreasing depth
    repaint();
  }
  
  /**
   * Gets status of view with no. index.
   */
  public boolean getViewActive(int index) {
    return this.viewActive[index];
  }
  
  /**
   * Gets status of all views.
   */
  public boolean[] getViewActive() {
    return this.viewActive;
  }
  
  /**
   * Sets status of view of no. index.
   */
  public void setViewActive(int index, boolean viewActive) {
    this.viewActive[index] = viewActive;
    repaint();
  }
  
  /**
   * Sets status of all views.
   * <P>The number of booleans must be equal or greater than the number of views
   * that are attached to this screen. The order of the booleans is in
   * decreasing depth, i.e. from the back to the front.
   */
  public void setViewActive(boolean[] viewActive) {
    this.viewActive = viewActive;
    repaint();
  }
}
