package physbeans.inout;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import java.beans.*;
import java.io.*;
import physbeans.inout.PhysicsPanel;
import physbeans.math.*;

/**
 * Bean for input and output of a real number via a text field<br>
 * value has a given output precision<br>
 * text field is surrounded by a description text and a unit.
 */
public class NumericTextField extends PhysicsPanel implements Serializable {
  protected int precision; // precision of displayed doubles
  protected int offset;    // minimal distance from tex field to right edge
  // used for justification of NumericalTextLine's
  
  protected String descriptionIcon, unitIcon;
  
  protected double lastValue;    // to go back in case of wrong input
  protected JTextField tf;
  protected JLabel descriptionLabel, unitLabel;
  protected Box.Filler filler;
  
  /**
   * constructs a default NumericTextField
   */
  public NumericTextField() {
    this(0.0, 8, 3, "Description", "Unit");
  }
  
  public NumericTextField(double val, int cols, int prec,
      String desc, String unit) {
    super();
    setLayout(new GridBagLayout());
    precision = prec;
    lastValue = 0.0;
    offset = 0;
    
    // setup description label
    descriptionLabel = new JLabel(desc, JLabel.RIGHT);
    descriptionLabel.setFont(null);
    descriptionLabel.setBorder(BorderFactory.createEmptyBorder(0,0,0,4));
    
    // setup text field
    tf = new JTextField("" + val, cols);
    tf.setBackground(Color.white);
    tf.setForeground(Color.black);
    tf.addActionListener(new MyTextFieldListener());
    
    // setup units label
    unitLabel = new JLabel(unit, JLabel.LEFT);
    unitLabel.setBorder(BorderFactory.createEmptyBorder(0,4,0,0));
    unitLabel.setFont(null);
    
    // build up GUI
    // unit comes into a vertical box, together with a strut of offset width
    // this allows justification of text lines
    Box unitBox = new Box(BoxLayout.Y_AXIS);
    Dimension d = new Dimension(offset, 0);
    filler = new Box.Filler(d, d, d);
    filler.setAlignmentX(Component.LEFT_ALIGNMENT);
    
    unitBox.add(unitLabel);
    unitBox.add(filler);
    add(descriptionLabel);
    add(tf);
    add(unitBox);
  }
  
  public void doLayout() {
    super.doLayout();
    tf.setFont(null); // AFTER definition of text or setting of value
    // best at layout time so that a font is already defined from the parent
    
  }
  
  /**
   * fire property change event<br>
   * access changed to public for NumericTextLine
   */
  public void firePropertyChange(String name, Object oldVal, Object newVal) {
    super.firePropertyChange(name, oldVal, newVal);
  }
  
  /**
   * fire vetoable change event<br>
   * access changed to public for NumericTextLine
   */
  public void fireVetoableChange(String name, Object oldVal, Object newVal)
  throws PropertyVetoException {
    super.fireVetoableChange(name, oldVal, newVal);
  }
  
  /**
   * gets number of columns of text field<br>
   */
  public int getColumns() {
    return tf.getColumns();
  }
  
  /**
   * gets description field of the numeric text field
   */
  public String getDescription() {
    return descriptionLabel.getText();
  }
  
  /**
   * gets precision of text field<br>
   */
  public int getPrecision() {
    return precision;
  }
  
  /**
   * gets unit field of the numeric text field
   */
  public String getUnit() {
    return unitLabel.getText();
  }
  
  /**
   * gets current value of text field. <br>
   *  assumes that value is valid
   */
  public double getValue() {
    String s = tf.getText();
    if (s.equals(Maths.infinityString)) {
      return Double.POSITIVE_INFINITY;
    } else {
      return Double.valueOf(s).doubleValue();
    }
  }
  
  /**
   * true if text field is editable
   */
  public boolean isEditable() {
    return tf.isEditable();
  }
  
  /**
   * gets number of columns of text field
   */
  public void setColumns(int col) {
    tf.setColumns(col);
  }
  
  /**
   * sets description field of the numeric text field
   */
  public void setDescription(String s) {
    descriptionLabel.setText(s);
  }
  
  /**
   * Gets filename of icon used for the description field.
   */
  public String getDescriptionIcon() {
    return descriptionIcon;
  }
  
  /**
   * Sets filename of icon used for the description field.
   */
  public void setDescriptionIcon(String file) {
    descriptionIcon = file;
    descriptionLabel.setIcon(getIcon(file));
    descriptionLabel.setVerticalTextPosition(SwingConstants.BOTTOM);
  }
  
  /**
   * Gets filename of icon used for the unit field.
   */
  public String getUnitIcon() {
    return unitIcon;
  }
  
  /**
   * Sets filename of icon used for the unit field.
   */
  public void setUnitIcon(String file) {
    unitIcon = file;
    unitLabel.setIcon(getIcon(file));
    unitLabel.setVerticalTextPosition(SwingConstants.BOTTOM);
  }
  
  
  /**
   * sets editable status of text field. <br>
   * true for input field, false for output field
   */
  public void setEditable(boolean flag) {
    tf.setEditable(flag);
  }
  
  /**
   * gets precision of text field<br>
   */
  public void setPrecision(int p) {
    precision = p;
  }
  
  /**
   * sets unit field of the numeric text field
   */
  public void setUnit(String s) {
    unitLabel.setText(s);
  }
  
  /**
   * sets TextField to value. <br>
   *  uses internal precision
   */
  public void setValue(double f) {
    try {
      // ask listeners whether new value ok
      fireVetoableChange("value", null, new Double(f));
      
      // no veto -> set text field and announce new value "officially"
      setValueInternally(f);
      firePropertyChange("value", null, new Double(f)); // inform listeners
    } catch (PropertyVetoException e) {
      // veto -> return to last value
      tf.setText("" + lastValue);
    }
  }
  
  /**
   * Sets TextField to value using internal precision.
   * Doesn't fire an event or ask the veto listeners.
   */
  public void setValueInternally(double val) {
    tf.setText(Maths.toString(val, precision));
    lastValue = getValue();     // value is rounded -> get again
  }
  
  /**
   * conveniance function: allows to use the first element of a vector
   * instead of a scalar for input
   */
  public void setValueFromVector(double[] v) {
    setValue(v[0]);
  }
  
  /**
   * conveniance function: allows to use an element of a vector
   * instead of a scalar for input
   */
  public void setValueFromVector(DVector v, int i) {
    setValue(v.get(i));
  }
  
  /**
   * gets size of offset at right margin. <br>
   * used for justification of number fields
   */
  public int getOffset() {
    return this.offset;
  }
  
  /**
   * sets size of offset at right margin. <br>
   * used for justification of number fields
   */
  public void setOffset(int offset) {
    this.offset = offset;
    adjustFiller();
  }
  
  /**
   * resizes the filler to adapt to offset size changes
   */
  protected void adjustFiller() {
    Dimension newSize = new Dimension(offset, 0);
    filler.changeShape(newSize, newSize, newSize);
    revalidate();
  }
  
  /**
   * find and load icons
   */
  protected ImageIcon getIcon(String file) {
    ClassLoader myCL = getClass().getClassLoader();
    URL myURL = myCL.getResource(file);
    if (myURL != null) {
      return new ImageIcon(myURL);
    } else {
      return null;
    }
  }
  
  /**
   * listener to text field action. <br>
   * enforces valid values<br>
   * informs PropertyChangeListeners
   */
  protected class MyTextFieldListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      try {
        double newValue = getValue();
        setValue(newValue);     // informs listeners
      } catch (NumberFormatException exception) {
        tf.setText("" + lastValue); // return to last value
      }
    }
  }
}
