package physbeans.inout;

import physbeans.model.*;
import physbeans.event.*;
import physbeans.views.*;
import physbeans.math.*;
import java.awt.*;


/**
 * XY Oscilloscope displaying several input values. The first value defines the
 * common x value, the other values define y values of the different lines.
 */
public class XYScope extends ControlledLayeredScreen {
  
  protected DataBuffer data;
  protected DataBufferView dataView;
  
  /**
   * Creates a new instance of MultiScope
   */
  public XYScope() {
    super();
    setScreenBackground(Color.BLACK);
    setBordered(true);
    
    createGUI();
    dataView.setShowMarker(false);
    
    data.addTriggerListener(new TriggerListener() {
      public void sourceChanged(TriggerEvent evt) {
        data1SourceChanged(evt);
      }
    });
  }
  
  /**
   * Gets number of lines to draw.
   */
  public int getNLines() {
    return data.getNBuffer() - 1;  // first one for x values
  }
  
  /**
   * Sets number of lines to draw.
   */
  public void setNLines(int nLines) {
    data.setNBuffer(nLines + 1);  // first one for x values
  }
  
  /**
   * Gets the size of the internal data buffer.
   */
  public int getBufferSize() {
    return data.getSize();
  }
  
  /**
   * Sets the size of the internal data buffer.
   */
  public void setBufferSize(int bufferSize) {
    data.setSize(bufferSize);
  }
  
  /**
   * Gets value of dotted flag.<br>
   * If true dots are drawn instead of lines.
   */
  public boolean isDotted() {
    return dataView.isDotted();
  }
  
  /**
   * Sets value of dotted flag.<br>
   * If true dots are drawn instead of lines.
   */
  public void setDotted(boolean dotted) {
    dataView.setDotted(dotted);
  }
  
  /**
   * Gets size of dots (in pixel) drawn in dotted mode.
   */
  public int getDotSize() {
    return dataView.getDotSize();
  }
  
  /**
   * Sets size of dots (in pixel) drawn in dotted mode.
   */
  public void setDotSize(int dotSize) {
    dataView.setDotSize(dotSize);
  }
  
  /**
   * Gets drawing status of a line.
   */
  public boolean getLineActive(int index) {
    return dataView.getLineActive(index);
  }
  
  /**
   * Gets drawing status of all lines.
   */
  public boolean[] getLineActive() {
    return dataView.getLineActive();
  }
  
  /**
   * Sets drawing status of a line.
   */
  public void setLineActive(int index, boolean lineActive) {
    dataView.setLineActive(index, lineActive);
  }
  
  /**
   * Sets drawing status of all lines.
   */
  public void setLineActive(boolean[] lineActive) {
    dataView.setLineActive(lineActive);
  }
  
  /**
   * Gets line widths of the lines to draw.
   */
  public float[] getLineWidths() {
    return dataView.getLineWidths();
  }
  
  /**
   * Sets line widths of the lines to draw.<br>
   * If a value is 0, the line is drawn as thin as possible.
   */
  public void setLineWidths(float[] lw) {
    dataView.setLineWidths(lw);
  }
  
  /**
   * Gets value of showMarker flag.<br>
   * If true, the latest data is displayed with a marker.
   */
  public boolean isShowMarker() {
    return dataView.isShowMarker();
  }
  
  /**
   * Sets value of showMarker flag.<br>
   * If true, the latest data is displayed with a marker.
   */
  public void setShowMarker(boolean showMarker) {
    dataView.setShowMarker(showMarker);
  }
  
  /**
   * Gets the color of a line.
   */
  public Color getLineColor(int index) {
    return dataView.getLineColor(index);
  }
  
  /**
   * Gets the colors of all lines.
   */
  public Color[] getLineColor() {
    return dataView.getLineColor();
  }
  
  /**
   * Sets the color of a line.
   */
  public void setLineColor(int index, Color lineColor) {
    dataView.setLineColor(index, lineColor);
  }
  
  /**
   * Sets the colors of all lines.
   */
  public void setLineColor(Color[] lineColor) {
    dataView.setLineColor(lineColor);
  }
  
  /**
   * Resets the internal data buffers and clears the screen.
   */
  public void restart() {
    data.restart();
  }
  
  /**
   * Sets new data values for screen and draws immediately.
   */
  public void setInputVector(DVector v) {
    data.storeData(v);
  }
  
  /**
   * Gets data values of buffer i.
   * Utility routine for test purposes.
   */
  public double[] getValues(int i) {
    return data.getData()[i].getDVector(0, data.getNData()-1).getArray();
  }
  
  /**
   * Creates GUI of the component.<br>
   * Isolated from constructor so that children can overwrite it.
   */
  protected void createGUI() {
    setScaleY(true);
    
    data = new DataBuffer();
    setBufferSize(1024);
    
    dataView = new DataBufferView();
    dataView.setModel(data);
    screen.addView(dataView);
  }
  
  // listener
  
  protected void data1SourceChanged(TriggerEvent evt) {
    dataView.setModel(data);
  }
}
