package physbeans.model;

import java.util.*;
import java.io.*;
import java.beans.*;
import physbeans.event.*;
import physbeans.math.*;

/**
 * model of a number of points given by two real coordinates<br>
 * points can be added incrementally<br>
 * Optionally one point can be "active".
 */
public class Point2dVector implements Serializable {
  
  /** list of points, contains 2d DVectors */
  protected Vector<DVector> points;
  
  /** index of "active" point */
  protected int activePoint;
  
  /**
   * creates an empty vector of points
   */
  public Point2dVector() {
    points = new Vector<DVector>();
    activePoint = -1;  // no active point
  }
  
  /**
   * Creates a standard vector of dim points.
   * Points are positioned along the x axis with equal distance of 1.
   */
  public Point2dVector(int dim) {
    this();
    
    for (int i = 0; i < dim; i++) {
      points.add(new DVector((double) i, 0.0));
    }
  }
  
  /**
   * creates a vector of points from arrays of x and y coordinates
   */
  public Point2dVector(double[] x, double[] y) {
    this();
    
    if (x != null && y != null) {
      int size = Math.min(x.length, y.length);
      for (int i = 0; i < size; i++) {
        points.add(new DVector(x[i], y[i]));
      }
    }
  }
  
  /**
   * creates a vector of points from DVectors of x and y coordinates
   */
  public Point2dVector(DVector x, DVector y) {
    this(x.getArray(), y.getArray());
  }
  
  /**
   * adds point at (x,y)
   */
  public void addPoint(double x, double y) {
    points.add(new DVector(x, y));
  }
  
  /**
   * removes all points
   */
  public void clear() {
    points.clear();
  }
  
  /**
   * Gets number of points.
   */
  public int size() {
    return points.size();
  }
  
  /**
   * Gets coordinates of point n.
   */
  public DVector getPoint(int n) {
    if (n >= 0 && n < size()) {
      return (DVector) points.get(n);
    } else {
      return new DVector(Double.NaN, Double.NaN);
    }
  }
  
  /**
   * Gets x coordinate of point n.
   */
  public double getX(int n) {
    return getPoint(n).x();
  }
  
  /**
   * Gets y coordinate of point n.
   */
  public double getY(int n) {
    return getPoint(n).y();
  }
  
  /**
   * sets coordinates of point n
   */
  public void setPoint(int n, DVector v) {
    if (n >= 0 && n < size()) {
      points.set(n, v);
    }
  }
  
  /**
   * gets index of "active" point.<br>
   * if negative then no point is active
   */
  public int getActivePoint() {
    return this.activePoint;
  }
  
  /**
   * sets index of "active" point.<br>
   * if negative then no point is active
   */
  public void setActivePoint(int activePoint) {
    this.activePoint = activePoint;
  }
  
  /**
   * gets DVector with x coordinates of all points
   */
  public DVector getXCoords() {
    int n = size();
    DVector v = new DVector(n);
    for (int i = 0; i < n; i++) {
      v.set(i, getPoint(i).get(0));
    }
    return v;
  }
  
  /**
   * gets DVector with y coordinates of all points
   */
  public DVector getYCoords() {
    int n = size();
    DVector v = new DVector(n);
    for (int i = 0; i < n; i++) {
      v.set(i, getPoint(i).get(1));
    }
    return v;
  }
  
  /**
   * Returns a (deep) copy of the current Point2dVector.
   */
  public Point2dVector copy() {
    return new Point2dVector(getXCoords(), getYCoords());
  }
}