package physbeans.phys;

import java.io.*;
import physbeans.math.*;

/**
 * Models a number of reversible thermodynamic state changes that build up
 * a circle.<br>
 * It is fixed by giving
 * <ul>
 *    <li>the number of corner points,</li>
 *    <li>the polytropic exponent of each state change,</li>
 *    <li>the start point (p0, V0),</li>
 *    <li>the intermediary volumes (or pressures for isochores),</li>
 *    <li>the number of steps for each state change.</li>
 * </ul>
 * The last volume is computed from the other values by the circle closing
 * requirement.
 */
public class CyclicProcess extends PhysModel implements Serializable {
  
  protected ReversibleStateChange[] stateChange;
  
  protected int nCur;
  protected int phase;
  
  protected final double V0 = 1.0e-3;
  protected final double P0 = 2.0e5;
  protected final int NSTEPS = 5;
  
  /** Creates a standard CyclicProcess with 4 corner states. */
  public CyclicProcess() {
    setNStates(4);
    
  }
  
  /**
   * Gets the number of corner states used in the cyclic process.
   */
  public int getNStates() {
    return stateChange.length;
  }
  
  /**
   * Sets the number of corner states used in the cyclic process.
   * Creates a default process made up of alternating isobaric and isochoric
   * parts. If the number of states is odd the closing step is n = 1 (i.e.
   * isothermal for an ideal gas).
   */
  public void setNStates(int len) {
    if (len < 3) {
      throw new IllegalArgumentException("number of states less than 3");
    }
    
    stateChange = new ReversibleStateChange[len];
    int[] nSteps = new int[len];
    for (int i = 0; i < len; i++) {
      nSteps[i] = NSTEPS;
      stateChange[i] = new ReversibleStateChange();
    }
    setNSteps(nSteps);
    
    double[] expos = new double[len];
    for (int i = 0; i < len; i+=2) {
      expos[i] = 0.0;
    }
    for (int i = 1; i < len; i+=2) {
      expos[i] = Double.POSITIVE_INFINITY;
    }
    if (len % 2 != 0) {
      expos[len-1] = 1.0;
    }
    setPolytropicExponents(new DVector(expos));
    
    setStartPressure(P0);
    double[] vols = new double[len-1];
    vols[0] = V0;
    for (int i = 1; i < len/2; i++) {
      vols[2*i-1] = (i+1)*V0;
      vols[2*i] = P0/(i+1);
    }
    if (len % 2 != 0) {
      vols[len-2] = (len/2 + 1)*V0;
    }
    setPoints(new DVector(vols));
  }
  
  /**
   * Gets the polytropic exponents of all state changes.
   */
  public DVector getPolytropicExponents() {
    int len = stateChange.length;
    DVector res = new DVector(len);
    
    for (int i = 0; i < len; i++) {
      res.set(i, stateChange[i].getN());
    }
    return res;
  }
  
  /**
   * Sets the polytropic exponents of all state changes.
   */
  public void setPolytropicExponents(DVector expos) {
    int len = stateChange.length;
    if (expos.getDimension() != len) {
      throw new IllegalArgumentException("wrong number of exponents");
    }
    
    // changing an exponent can change the interpretation of the points:
    // e.g. if n changes from infinity to something else, the meaning of its end
    // point changes from pressure to volume.
    // This creates a problem when configuring a CyclicProcess as a bean, since
    // the order of settings depends on the GUI builder. Therefore the current
    // point values are stored and reapplied afterwards.
    DVector points = getPoints();
    for (int i = 0; i < len; i++) {
      stateChange[i].setN(expos.get(i));
    }
    setPoints(points);
  }
  
  /**
   * Changes the value of polytropic exponent nr. i.
   * Conveniance function to simplify routing.
   */
  public void changePolytropicExponent(int i, double val) {
    DVector expos = getPolytropicExponents();
    expos.set(i, val);
    setPolytropicExponents(expos);
  }
  
  /**
   * Gets the start pressure in Pa.
   */
  public double getStartPressure() {
    return stateChange[0].getStartPressure();
  }
  
  /**
   * Sets the start pressure in Pa.
   */
  public void setStartPressure(double p0) {
    stateChange[0].setStartPressure(p0);
    
    int len = stateChange.length;
    if (stateChange[len-1].isIsochoric()) {
      stateChange[len-1].setFinalPressure(p0);
    }
    
    computeInternalPoints();
    restart();
  }
  
  /**
   * Gets volumes (or pressures for isochorics) of the corner states except for
   * the last one, which is computed internally.
   */
  public DVector getPoints() {
    int len = stateChange.length;
    DVector res = new DVector(len-1);
    
    res.set(0, stateChange[0].getStartVolume());
    for (int i = 1; i < len-1; i++) {
      res.set(i, stateChange[i-1].getFinalPoint());
    }
    return res;
  }
  
  /**
   * Sets the values of the points that define the cycle.
   * Values are volumes except for end points of isochorics.
   * The input vector has to be one shorter than the number of corner
   * states, because the last point is computed from the other values.
   */
  public void setPoints(DVector vols) {
    int len = stateChange.length;
    if (vols.getDimension() != len - 1) {
      throw new IllegalArgumentException("wrong number of steps");
    }
    
    // point 0
    double val = vols.get(0);
    stateChange[0].setStartVolume(val);
    if (! stateChange[len-1].isIsochoric()) {
      stateChange[len-1].setFinalVolume(val);
    }
    
    // points 1 .. N-2
    for (int i = 1; i < len-1; i++) {
      val = vols.get(i);
      stateChange[i-1].setFinalPoint(val);
      if (stateChange[i-1].isIsochoric()) {
        stateChange[i].setStartPressure(val);
      } else {
        stateChange[i].setStartVolume(val);
      }
    }
    
    computeInternalPoints();
    restart();
  }
  
  /**
   * Gets the number of steps for each state change.
   */
  public int[] getNSteps() {
    int len = stateChange.length;
    int[] res = new int[len];
    
    for (int i = 0; i < len; i++) {
      res[i] = stateChange[i].getNSteps();
    }
    return res;
  }
  
  /**
   * Sets the number of steps for each state change.
   */
  public void setNSteps(int[] nSteps) {
    int len = stateChange.length;
    if (nSteps.length != len) {
      throw new IllegalArgumentException("array length not equal to number of steps");
    }
    
    for (int i = 0; i < len; i++) {
      stateChange[i].setNSteps(nSteps[i]);
    }
  }
  
  /**
   * Gets the number of the current phase.
   * Returns the value as a double to make it compatible with the usual signal
   * routing.
   */
  public double getPhase() {
    return (double) phase;
  }
  
  
  
  /**
   * Gets the system to which the state change is applied.
   * @return the thermodynamic system used.
   */
  public ThermodynamicSystem getSystem() {
    return stateChange[0].getSystem();
  }
  
  /**
   * Sets the system to which the state change is applied.
   * @param system New thermodynamic system to use.
   */
  public void setSystem(ThermodynamicSystem system) {
    system.setAutoTriggered(false);    // the CyclicProcess controls triggering
    for (int i = 0; i < stateChange.length; i++) {
      stateChange[i].setSystem(system);
    }
    restart();
  }
  
  /**
   * Sets system to start values and prepares stepping.
   */
  public void restart() {
    int len = stateChange.length;
    
    // cycle through the states in inverse order, so that the first one comes
    // last. This guarantees that the (only!) system has the correct start
    // condition of the cycle, i.e. start volume and pressure of stateChange[0].
    for (int i = stateChange.length-1; i >= 0; i--) {
      stateChange[i].restart();
    }
    
    ThermodynamicSystem system = getSystem();
    if (system != null) {
      system.restart();
      system.trigger();
    }
    
    nCur = 0;
    phase = 0;
    trigger();
  }
  
  /**
   * Proceeds the associated thermodynamical system by one step through the
   * cycle.<br>
   * The last point of a state change and the first point of the next one are
   * both sent, though they correspond to the same state.
   */
  public void nextStep() {
    if (nCur == 0) {
      stateChange[phase].restart();
    }
    
    stateChange[phase].nextStep();
    
    nCur++;
    if (nCur == stateChange[phase].getNSteps() + 1) {
      nCur = 0;
      phase = (phase+1) % stateChange.length;
      trigger();
    }
    getSystem().trigger();
  }
  
  /**
   * Computes the undefined values of internal points and stores them in
   * the state changes.
   * These values are:
   *     nothing for point 0  (is completely defined)
   *     pressures for points 1 .. N-2 (resp. volumes, if state(i-1) = isochoric)
   *     pressure and volume for point N-1
   */
  protected void computeInternalPoints() {
    int len = stateChange.length;
    if (len < 2) {
      return;    // nothing to do
    }
    
    // the state of the system connected to the cyclic process should not be
    // changed! Save it now and restore it at the end.
    ThermodynamicSystem system = stateChange[0].getSystem();
    double pSystem = 0.0;
    double vSystem = 0.0;
    if (system != null) {
      pSystem = system.getStateValue(ThermodynamicStateVariable.P);
      vSystem = system.getStateValue(ThermodynamicStateVariable.V);
    }
    
    for (int i = 1; i < len-1; i++) {
      if (stateChange[i-1].isIsochoric()) {
        double vEnd = stateChange[i-1].getStartVolume();
        stateChange[i].setStartVolume(vEnd);
      } else {
        double pStart = stateChange[i-1].getStartPressure();
        double vStart = stateChange[i-1].getStartVolume();
        double vEnd = stateChange[i-1].getFinalVolume();
        double n = stateChange[i-1].getN();
        double pEnd = pStart*Math.pow(vStart/vEnd, n);
        stateChange[i].setStartPressure(pEnd);
      }
    }
    
    // computation of last point as intersection of two polytropes
    double p0 = stateChange[0].getStartPressure();
    double v0 = stateChange[0].getStartVolume();
    double p2 = stateChange[len-2].getStartPressure();
    double v2 = stateChange[len-2].getStartVolume();
    
    // all 4 cases have to be dealt with seperately
    double v1, p1;
    double n1 = stateChange[len-1].getN();
    double n2 = stateChange[len-2].getN();
    if (stateChange[len-2].isIsochoric()) {
      if(stateChange[len-1].isIsochoric()) {
        // two consecutive isochorics -> just use the last end point
        v1 = v2;
        p1 = p2;
      } else {
        v1 = v2;
        p1 = p0*Math.pow(v0/v1, n1);
      }
      stateChange[len-2].setFinalPressure(p1);
    } else {
      if(stateChange[len-1].isIsochoric()) {
        v1 = v0;
        p1 = p2*Math.pow(v2/v1, n2);
      } else {
        v1 = p0/p2 * Math.pow(v0, n1)/Math.pow(v2,n2);
        v1 = Math.pow(v1, 1.0/(n1-n2));
        p1 = p0*Math.pow(v0/v1, n1);
      }
      stateChange[len-2].setFinalVolume(v1);
    }
    
    stateChange[len-1].setStartPressure(p1);
    stateChange[len-1].setStartVolume(v1);
    
    // restore system state
    if (system != null) {
      system.setState(pSystem, vSystem);
    }
  }
}
