package physbeans.phys;

import physbeans.func.*;
import physbeans.math.*;

/**
 * A class describing an n-dimensional linear oscillator with friction
 * and external forces given by<br>
 *    M x'' + B x' + C x = F(t)<br>
 * where position x and force F are n-dimensional vectors and M, B, C are
 * symmetric nxn matrices defining the masses, dampers and springs.
 *<P>This class is generic, it provides no means to define the system matrices.
 * This is done in different ways by its children.
 */
public class GenericCoupledSprings extends SolveODEFunction {

  protected Matrix m;
  protected Matrix c;
  protected Matrix b;
  protected Generic1dFunction externalForceFunction;

  /**
   * Creates three-dimensional standard CoupledSprings.
   */
  public GenericCoupledSprings() {
    super();

    m = Matrix.identity(3, 3);
    c = new Matrix(new double[]{2, -1, 0, -1, 2, -1, 0, -1, 2}, 3);
    b = new Matrix(3, 3);

    setOde(new CoupledSpringsODE());
    externalForceFunction = null; // by default, force = 0
    restart();
  }

  /**
   * Gets mass matrix m.
   */
  public Matrix getM() {
    return this.m;
  }

  /**
   * Gets stiffness matrix c.
   */
  public Matrix getC() {
    return this.c;
  }

  /**
   * Gets viscous damping matrix b.
   */
  public Matrix getB() {
    return this.b;
  }

  /**
   * Gets vector of initial positions.
   */
  public DVector getX0() {
    DVector xv0 = getInitialValues();
    int vStart = xv0.getDimension() / 2;
    return xv0.getDVector(0, vStart - 1);
  }

  /**
   * Sets vector of initial positions.
   */
  public void setX0(DVector x0) {
    DVector xv0 = getInitialValues();
    int n = x0.getDimension();
    if (xv0.getDimension() != 2 * n) {
      xv0 = new DVector(2 * n);
    }
    xv0.setDVector(0, n - 1, x0);
    setInitialValues(xv0);
  }

  /**
   * Gets vector of initial velocities.
   */
  public DVector getV0() {
    DVector xv0 = getInitialValues();
    int vStart = xv0.getDimension() / 2;
    return xv0.getDVector(vStart, xv0.getDimension() - 1);
  }

  /**
   * Sets vector of initial velocities.
   */
  public void setV0(DVector v0) {
    DVector xv0 = getInitialValues();
    int n = v0.getDimension();
    if (xv0.getDimension() != 2 * n) {
      xv0 = new DVector(2 * n);
    }
    xv0.setDVector(n, 2 * n - 1, v0);
    setInitialValues(xv0);
  }

  /**
   * Gets function that computes time depending values of the external force
   */
  public Generic1dFunction getExternalForceFunction() {
    return this.externalForceFunction;
  }

  /**
   * Sets function that computes time depending values of the external force
   */
  public void setExternalForceFunction(Generic1dFunction externalForceFunction) {
    this.externalForceFunction = externalForceFunction;
  }

  /**
   * Gets current values of the displacements.
   * <P>Conveniance function to simplify routing.
   */
  public DVector getDisplacements() {
    DVector xv = getOutputVector();
    return xv.getDVector(0, xv.getDimension() / 2 - 1);
  }

  /**
   * Gets current values of the velocities.
   * <P>Conveniance function to simplify routing.
   */
  public DVector getVelocities() {
    DVector xv = getOutputVector();
    int dim = xv.getDimension() / 2;
    return xv.getDVector(dim, 2 * dim - 1);
  }

  /**
   * Gets current values of the accelerations.
   */
  public DVector getAccelerations() {
    DVector xv = getOutputVector();
    int dim = xv.getDimension() / 2;
    DVector x = xv.getDVector(0, dim - 1);
    DVector v = xv.getDVector(dim, 2 * dim - 1);

    // get external force as value of the externalForceFunction
    DVector force;
    if (externalForceFunction != null) {
      force = externalForceFunction.computeFunction(getTime());
    } else {
      force = new DVector(dim);
    }

    DVector result = force.copy();
    result.minusEquals(v.times(b));
    result.minusEquals(x.times(c));
    result = result.times(m.inverse());

    return result;
  }

  class CoupledSpringsODE extends ODE {

    public CoupledSpringsODE() {
      super(new DVector(2 * m.getColumnDimension()), 0.0);
    }

    public DVector f(DVector xv, double t) {
      int dim = xv.getDimension() / 2;
      DVector result = new DVector(2 * dim);

      // get external force as value of the externalForceFunction
      DVector force;
      if (externalForceFunction != null) {
        force = externalForceFunction.computeFunction(t);
      } else {
        force = new DVector(dim);
      }

      DVector x = xv.getDVector(0, dim - 1);
      DVector v = xv.getDVector(dim, 2 * dim - 1);

      // x' = v
      result.setDVector(0, dim - 1, v);

      // v' = M^-1*(F - B*v - C*x))
      DVector dv = force.minus(v.times(b));
      dv.minusEquals(x.times(c));
      dv = dv.times(m.inverse());
      result.setDVector(dim, 2 * dim - 1, dv);

      return result;
    }
  }
}
