package physbeans.phys;

import java.io.*;
import physbeans.model.*;
import physbeans.math.*;

/**
 * Optical lens defined by the intersection of two spherical surfaces and
 * an index of refraction.<br>
 * Only biconcave or biconvex lenses are considered. The outer medium is assumed
 * to have an index of refraction equal to 1.<br>
 * Ray propagation uses the refaction law at the lens surfaces. For imaging
 * purposes (focal points, image computation) the thick lens approximation is
 * used.
 */
public class Lens extends ImagingElement implements Serializable {
  
  protected double r1;
  protected double r2;
  protected double d;
  protected double h;
  protected double n;
  
  /** Creates a new instance of Lens */
  public Lens() {
    super();
    r1 = 10.0;
    r2 = -10.0;
    d = 5.0;
    h = 5.0;
    n = 1.459;
  }
  
  /**
   * Returns true, if lens is (bi-)convex.
   */
  public boolean isConvex() {
    return ((r1 > 0) && (r2 < 0));
  }
  
  /**
   * Returns true, if lens is (bi-)concave.
   */
  public boolean isConcave() {
    return ((r1 < 0) && (r2 > 0));
  }
  
  /**
   * Gets 1. radius of curvature.<br>
   * r1 is positiv if the center of the circle is to the right of the lens,
   * negativ otherwise.<br>
   * For a biconvex lens we have r1 > 0, r2 < 0,
   * for a biconcave lens r1 < 0, r2 > 0.
   */
  public double getRadius1() {
    return r1;
  }
  
  /**
   * Sets 1. radius of curvature
   */
  public void setRadius1(double r1) {
    this.r1 = r1;
    trigger();
  }
  
  /**
   * Gets 2. radius of curvature
   */
  public double getRadius2() {
    return r2;
  }
  
  /**
   * Sets 2. radius of curvature.
   */
  public void setRadius2(double r2) {
    this.r2 = r2;
    trigger();
  }
  
  /**
   * Gets thickness of the lens.
   */
  public double getThickness() {
    return d;
  }
  
  /**
   * Sets thickness of the lens.
   * It is bounded by the diameter of the smaller circle.
   */
  public void setThickness(double dNeu) {
    d = dNeu;
    if (isConvex()) {
      d = Math.min(d, 2.0*Math.min(r1, -r2)) - Maths.getAccuracy();
    }
    trigger();
  }
  
  /**
   * Gets real height of the lens (half diameter). <br>
   * For a concave lens this is simply the given maximal height.
   * For a convex lens it is bounded by intersection of the two spheres.
   */
  public double getHeight() {
    double hOut = h;
    if (isConvex()) {
      double x1 = getCurvatureCenter1() - pos;
      double hMax = Math.sqrt(r1*r1 - x1*x1);
      
      
      if (x1 < 0.0) {
        hMax = Math.min(r1, -r2);
      }
      
      hOut = Math.min(hOut, hMax);
    } else if (isConcave()) {
      hOut = Math.min(hOut, Math.min(-r1, r2));
    }
    return hOut;
  }
  
  /**
   * Gets maximal height of the lens.
   * In the case of a convex lens, the real height is bounded above by the diameter
   * of the intersection volume of the two spheres defining the lens.
   */
  public double getMaxHeight() {
    return h;
  }
  
  /**
   * Sets maximal height of the lens.
   * In the case of a convex lens, the height is bounded above by the diameter
   * of the intersection volume of the two spheres defining the lens.
   */
  public void setMaxHeight(double hNew) {
    h = hNew;
    trigger();
  }
  
  /**
   * Gets index of refraction of the lens material.
   */
  public double getIndexOfRefraction() {
    return n;
  }
  
  /**
   * Sets index of refraction of the lens material.
   */
  public void setIndexOfRefraction(double indexOfRefraction) {
    n = indexOfRefraction;
    trigger();
  }
  
  
  /**
   * Gets x coordinate of first curvature center.
   * convex -> x1 > pos,
   * concave -> x1 < pos.
   */
  public double getCurvatureCenter1() {
    if (isConvex()) {
      return 0.5*(r1 - r2 - d - (r2*r2 - r1*r1)/(r1 - r2 - d)) + pos;
    } else {
      // assumed to be concave
      return r1 - 0.5*d + pos;
    }
  }
  
  /**
   * Gets x coordinate of second curvature center.
   * convex -> x2 < pos,
   * concave -> x2 > pos.
   */
  public double getCurvatureCenter2() {
    if (isConvex()) {
      return -0.5*(r1 - r2 - d + (r2*r2 - r1*r1)/(r1 - r2 - d)) + pos;
    } else {
      // assumed to be concave
      return r2 + 0.5*d + pos;
    }
  }
  
  /**
   * Returns the smallest x value to which the first mirror extends.
   */
  public double getLeftEdge1() {
    if (isConvex()) {
      return getCurvatureCenter1() - r1;
    } else {
      double h1 = getHeight();
      return getCurvatureCenter1() + Math.sqrt(r1*r1 - h1*h1);
    }
  }
  
  /**
   * Returns the largest x value to which the first mirror extends.
   */
  public double getRightEdge1() {
    if (isConvex()) {
      if (Maths.isStrictlyPositive(h - getHeight())) {
        return pos;
      } else {
        double h1 = getHeight();
        return getCurvatureCenter1() - Math.sqrt(r1*r1 - h*h);
      }
    } else {
      return -d/2.0 + pos;
    }
  }
  
  /**
   * Returns the smallest x value to which the second mirror extends.
   */
  public double getLeftEdge2() {
    if (isConvex()) {
      if (Maths.isStrictlyPositive(h - getHeight())) {
        return pos;
      } else {
        return getCurvatureCenter2() + Math.sqrt(r2*r2 - h*h);
      }
    } else {
      return d/2.0 + pos;
    }
  }
  
  /**
   * Returns the largest x value to which the second mirror extends.
   */
  public double getRightEdge2() {
    if (isConvex()) {
      return getCurvatureCenter2() - r2;
    } else {
      double h1 = getHeight();
      return getCurvatureCenter2() - Math.sqrt(r2*r2 - h1*h1);
    }
  }
  
  
  /**
   * Extends a light ray through the optical element.
   */
  protected void extend(LightRay ray) {
    
    DVector pLast = ray.getLastPoint();
    double al = ray.getAngle();
    DVector pHit = new DVector(2);
    
    int rc = findHitPoint(pLast, al, pHit);
    
    DVector inVec = new DVector(Math.cos(al), Math.sin(al));
    DVector nVec = new DVector(2);
    double nIn = 1.0;     // default: entering the lens
    double nOut = n;
    
    Circle2D circle;
    switch (rc) {
      case 0:     // no hit
        return;
      case 1:     // refraction at first circle
        circle = new Circle2D(new DVector(getCurvatureCenter1(), 0.0), r1);
        nVec = circle.getNormal(pHit);
        if (r1 < 0) {
          nVec.timesEquals(-1.0);     // normal points outwards
        }
        if (nVec.times(inVec) > 0)  {     // leaving the lens
          nIn = n;
          nOut = 1.0;
        }
        break;
      case 2:     // refraction at second circle
        circle = new Circle2D(new DVector(getCurvatureCenter2(), 0.0), r2);
        nVec = circle.getNormal(pHit);
        if (r2 > 0) {
          nVec.timesEquals(-1.0);     // normal points outwards
        }
        if (nVec.times(inVec) > 0)  {     // leaving the lens
          nIn = n;
          nOut = 1.0;
        }
        break;
      case 3:    // refraction at upper plane
        nVec.set(1, 1.0);
        if (al > 0)  {     // leaving the lens
          nIn = n;
          nOut = 1.0;
        }
        break;
      case 4:    // refraction at lower plane
        nVec.set(1, -1.0);
        if (al < 0)  {     // leaving the lens
          nIn = n;
          nOut = 1.0;
        }
        break;
      default:
        throw new RuntimeException("illegal return code " + rc);
    }
    
    ray.addPoint(pHit);
    ray.setAngle(OpticalSurface.getRefractionAngle(inVec, nVec, nIn, nOut));
    propagate(ray);
    
  }
  
  /**
   * Computes the next hit point of the ray (p, al) and returns it in pHit.
   * Additionally returns a number denoting the hit surface:
   *   <ul>
   *     <li>0: no Hit</li>
   *     <li>1: left circle segment</li>
   *     <li>2: right circle segment</li>
   *     <li>3: upper line segment</li>
   *     <li>4: lower line segment</li>
   *  </ul>
   * If no hit exists (rc = 0) then the value of pHit is unchanged.
   */
  protected int findHitPoint(DVector p, double al, DVector pHit) {
    int rc = 0;      // no hit until the opposite is proven
    
    // begin with first circle
    DVector c1 = new DVector(getCurvatureCenter1(), 0.0);
    double xl1 = getLeftEdge1();
    double xr1 = getRightEdge1();
    Circle2D circle = new Circle2D(c1, r1, xl1, xr1);
    Ray2D lastRay = new Ray2D(p, al);
    double l = circle.getSegmentRayHitParameter(lastRay);
    if (!Double.isNaN(l)) {   // lastRay hits c1
      rc = 1;
    }
    
    // check second circle
    DVector c2 = new DVector(getCurvatureCenter2(), 0.0);
    double xl2 = getLeftEdge2();
    double xr2 = getRightEdge2();
    circle = new Circle2D(c2, r2, xl2, xr2);
    double lNew = circle.getSegmentRayHitParameter(lastRay);
    if (!Double.isNaN(lNew) && ((lNew < l) || Double.isNaN(l))) {  // lastRay hits c2 earlier
      l = lNew;
      rc = 2;
    }
    
    if (isConcave() || !Maths.isStrictlyPositive(h - getHeight())) {  // horizontal boundaries exist
      // check upper surface l1
      double xl = isConvex() ? xr1 : xl1;
      double xr = isConvex() ? xl2 : xr2;
      LineSegment2D upperLine = new LineSegment2D(new DVector(xl, h), new DVector(xr, h));
      lNew = upperLine.getRayHitParameter(lastRay);
      if (!Double.isNaN(lNew) && ((lNew < l) || Double.isNaN(l))) {  // ray hits l1 earlier
        l = lNew;
        rc = 3;
      }
      
      // check lower surface l2
      LineSegment2D lowerLine = new LineSegment2D(new DVector(xl, -h), new DVector(xr, -h));
      lNew = lowerLine.getRayHitParameter(lastRay);
      if (!Double.isNaN(lNew) && ((lNew < l) || Double.isNaN(l))) {  // ray hits l2 earlier
        l = lNew;
        rc = 4;
      }
    }
    
    if (rc != 0) {                 // a hit exists
      pHit.setDVector(0, 1, lastRay.getPointAtParameter(l));
    }
    
    return rc;
  }
  
  /**
   * Returns the position of the first principal plane.
   */
  public double getFirstPrincipalPlane() {
    if (isConvex()) {
      return getLeftEdge1() + getFirstPrincipalPlaneDistance();
    } else {
      return getRightEdge1() + getFirstPrincipalPlaneDistance();
    }
    
  }
  
  /**
   * Returns the position of the second principal plane.
   */
  public double getSecondPrincipalPlane() {
    if (isConvex()) {
      return getRightEdge2() + getSecondPrincipalPlaneDistance();
    } else {
      return getLeftEdge2() + getSecondPrincipalPlaneDistance();
    }
  }
  
  /**
   * Gets focal length of lens.
   * Uses equations for a thick lens.
   */
  public double getFocalLength() {
    return n*r1*r2 / ((n-1)*(n*(r2-r1) + (n-1)*d));
  }
  
  /**
   * Returns the position of the focal points as a Point2dVector.
   * Uses equations for a thick lens.
   */
  public Point2dVector getFocalPoints() {
    Point2dVector point = new Point2dVector();
    double fp = getFocalLength();
    point.addPoint(getFirstPrincipalPlane() - fp, 0.0);
    point.addPoint(getSecondPrincipalPlane() + fp, 0.0);
    return point;
  }
  
  /**
   * Computes the image of the object src in a thick lens approximation.<br>
   * Its rays connect the image point with the lens and are either part of the
   * source rays (for real images) or are backwards extensions of the source
   * rays (for virtual images).
   */
  public LightSource getImage(ObjectLightSource src) {
    GenericLightSource image = new GenericLightSource();
    LightRayBundle rays = new LightRayBundle();
    
    DVector srcPos = src.getCoords();
    DVector imgPos = getImagePoint(srcPos);
    image.setCoords(imgPos);
    
    // compute rays from the image to the mirror
    int nRays = src.getRays().getNRays();
    DVector angles = src.getStartAngles();
    for (int i = 0; i < nRays; i++) {
      LightRay ray = new LightRay(imgPos, Double.NaN);
      double ys = getHittingHeight(srcPos, angles.get(i));
      if (Math.abs(ys) <= getHeight()) {
        ray.addPoint(getSecondPrincipalPlane(), ys);
      }
      rays.addRay(ray);
    }
    
    image.setRays(rays);
    return image;
  }
  
  /**
   * Returns the distance of the two principal planes.
   */
  protected double getPrincipalPlanesWidth() {
    return (r1-r2-d)*(n-1)*d / ((n*(r1-r2) - (n-1)*d));
  }
  
  /**
   * Returns the distance of the first principal plane from the left edge.
   * The sign is given according to the usual convention.
   */
  protected double getFirstPrincipalPlaneDistance() {
    return r1*d / ((n*(r1-r2) - (n-1)*d));
  }
  
  /**
   * Returns the distance of the second principal plane from the right edge.
   * The sign is given according to the usual convention.
   */
  protected double getSecondPrincipalPlaneDistance() {
    return r2*d / ((n*(r1-r2) - (n-1)*d));
  }
  
  /**
   * Computes the height at which a ray, given by a start point and an angle,
   * hits the principal planes.
   */
  protected double getHittingHeight(DVector p, double angle) {
    return p.y() + (getFirstPrincipalPlane() - p.x())*Math.tan(angle);
  }
  
  /**
   * Computes the position of the image of an object at position (xSrc, ySrc).
   */
  protected DVector getImagePoint(DVector p) {
    double a = p.x() - getFirstPrincipalPlane();
    double fp = getFocalLength();
    fp = (a>0)? -fp: fp;
    double ap = 1.0/(1.0/fp + 1.0/a);
    return new DVector(getSecondPrincipalPlane() + ap, p.y()*ap/a);
  }
  
}
