package physbeans.phys;

import java.util.*;
import physbeans.math.*;

/**
 * Master class that contains all light sources and optical elements.
 * It propagates the light rays of all sources through the whole
 * optical system including multiple reflections between elements.
 */
public class OpticalBench {
  
  protected Vector<OpticalElement> opticalElements;
  protected Vector<LightSource> lightSources;
  
  /** Creates a new instance of OpticalBench */
  public OpticalBench() {
    opticalElements = new Vector<OpticalElement>();
    lightSources = new Vector<LightSource>();
  }
  
  /**
   * Propagates a light ray through all optical elements.
   */
  protected void propagate(LightRay ray) {
    propagate(ray, getLeftElement(ray));
  }
  
  
  /**
   * Propagates a light ray between optical elements left and left+1 through
   * the next element (depending on the ray's direction) and recurses through the
   * whole system.
   */
  protected void propagate(LightRay ray, int left) {
    if (Double.isNaN(ray.getAngle())) {
      return;
    }
    
    if (ray.getNPoints() == ray.getMaxPoints()) {
      ray.setAngle(Double.NaN);
      return;
    }
    
    if (ray.isRightGoing()) {
      if (left == opticalElements.size() - 1) { // reached right edge
        return;
      }
      OpticalElement el = opticalElements.get(left + 1);
      el.propagate(ray);
      if (ray.isRightGoing()) {
        propagate(ray, left + 1);
      } else {
        propagate(ray, left);
      }
    } else {   // ray is left going
      if (left == -1) { // reached left edge
        return;
      }
      OpticalElement el = opticalElements.get(left);
      el.propagate(ray);
      if (ray.isRightGoing()) {
        propagate(ray, left);
      } else {
        propagate(ray, left - 1);
      }
    }
  }
  
  /**
   * Returns the number of the next optical element to the left of the ray.
   * If the ray is to the left of the whole system it returns -1.
   */
  protected int getLeftElement(LightRay ray) {
    double xr = ray.getLastPoint().x();
    
    int i;
    for (i = 0; i < opticalElements.size(); i++) {
      OpticalElement el = opticalElements.get(i);
      if (!Maths.isStrictlyPositive(xr - el.getPosition())) {
        // allow for epsilon errors in double arithmetic!
        break;
      }
    }
    
    return i-1;
  }
  
  /**
   * Updates the light rays of the light source src by propagating them through
   * all optical elements.
   */
  protected void update(LightSource src) {
    LightRayBundle rays = src.getNewRays();
    for (int i = 0; i < rays.getNRays(); i++) {
      propagate(rays.getRay(i));
    }
  }
  
  /**
   * Updates the light rays of all light sources by propagating them through
   * all optical elements.
   */
  protected void updateAll() {
    for (int i = 0; i < lightSources.size(); i++) {
      update(lightSources.get(i));
    }
  }
  
  /**
   * Adds a light source to the optical bench. 
   */
  public void addLightSource(LightSource src) {
    if (!lightSources.contains(src)) {
      lightSources.add(src);
    }
    
    update(src);
  }
  
  /**
   * Adds an optical element to the optical bench.
   */
  public void addOpticalElement(OpticalElement el) {
    if (!opticalElements.contains(el)) {
      opticalElements.add(el);
    }
    // sort each time, since the position of el could have changed
    Collections.sort(opticalElements);
    
    updateAll();
  }
}
