package physbeans.phys;

import java.io.*;
import java.util.*;
import physbeans.func.Generic1dFunction;
import physbeans.math.*;
import physbeans.model.*;


/**
 * 2-stage random decay function.<br>
 * Computes statistically correct random number of particles for an
 * exponential decay chain with three particle types (the last one being stable).<br>
 * assumes initial conditions
 *  N1(0) = N0, N2(0) = N3(0) = 0<br>
 */
public class ParticleDecayChain extends Generic1dFunction implements Serializable {
  
  protected StateVector particles;
  
  protected double lambda1, lambda2;
  protected boolean reproducible;
  
  protected double tLast;
  protected Random rand;
  
  /** creates ExponentialDecayFunction with standard values */
  public ParticleDecayChain() {
    super(3);
    
    particles = new StateVector();
    particles.setNumberOfElements(1000);
    
    lambda1 = 1.0;
    lambda2 = 1.0;
    tLast = 0.0;
    reproducible = false;
    rand = new Random();
    
    initStates();
  }
  
  /**
   * computes the next values at time t
   */
  public DVector computeFunction(double t) {
    // explicit formulas have formal singularities for equal lambda
    // I use the resulting limiting functions for almost equal lambda
    
    double dt = t - tLast;
    
    if (dt > 0) { // otherwise just restarted, need another time point
      double q1 = Math.exp(-lambda1 * dt);
      double q2 = Math.exp(-lambda2 * dt);
      double rnd;
      
      for (int i = 0; i < particles.getNumberOfElements(); i++) {
        // loop over all particles
        switch (particles.getState(i)) {
          case 0 :
            rnd = rand.nextDouble();
            
            // special case lambda1 = lambda2
            if (Maths.isEqual(lambda1, lambda2)) {
              if (rnd < 1 - (1 + lambda1*dt) * q1) { // decay 0->2
                particles.setState(i, 2);
              } else if (rnd < 1 - q1) { // decay 0->1
                particles.setState(i, 1);
              }
              break;
            }
            
            // generic case: lambda1 != lambda2
            double c1 = lambda1 / (lambda1 - lambda2);
            if (rnd < 1 - (1 - c1) * q1 - c1 * q2) { // decay 0->2
              particles.setState(i, 2);
            } else if (rnd < 1 - q1) { // decay 0->1
              particles.setState(i, 1);
            }
            break;
            
          case 1 :
            rnd = rand.nextDouble();
            if (rnd > q2) { // decay 1->2
              particles.setState(i, 2);
            }
            break;
            
          case 2 :
            // do nothing, particle is stable
            break;
          default :
            throw new IllegalArgumentException("illegal particle state "
                + particles.getState(i));
        }
      }
    }
    
    // prepare for next value
    tLast = t;
    
    int[] count = particles.getStateCounts();
    DVector out = new DVector(3);
    out.set(0, count[0]);
    out.set(1, count[1]);
    out.set(2, count[2]);
    return out;
  }
  
  /**
   * Resets states at restart.
   */
  public void restart() {
    initStates();
    trigger();
  }
  
  /**
   * gets half life time of particle type 1
   */
  public double getTHalf1() {
    return Math.log(2)/lambda1;
  }
  
  /**
   * sets half life time of particle type 1
   */
  public void setTHalf1(double tHalf1) {
    lambda1 = Math.log(2)/tHalf1;
  }
  
  /**
   * gets half life time of particle type 2
   */
  public double getTHalf2() {
    return Math.log(2)/lambda2;
  }
  
  /**
   * sets half life time of particle type 2
   */
  public void setTHalf2(double tHalf2) {
    lambda2 = Math.log(2)/tHalf2;
  }
  
  /**
   * Gets initial number of particles (all of type 0)
   */
  public int getN0() {
    return particles.getNumberOfElements();
  }
  
  /**
   * Sets initial number of particles (all of type 0)
   */
  public void setN0(int n0) {
    particles.setNumberOfElements(n0);
    
    initStates();
    trigger();
  }
  
  /**
   * If true, each restart creates the same sequence of random numbers.
   */
  public boolean isReproducible() {
    return reproducible;
  }
  
  /**
   * If true, the random number generator starts with a fixed seed after a reset,
   * else the internal clock is used to produce different values with each run.
   */
  public void setReproducible(boolean newState) {
    reproducible = newState;
    if (reproducible) {
      rand.setSeed(0);
    } else {
      rand = new Random();
    }
  }
  
  /**
   * Returns vector of particle states.
   */
  public StateVector getStates() {
    return particles;
  }
  
  /**
   * init values
   */
  protected void initStates() {
    particles.restart();
    
    if (reproducible) {
      rand.setSeed(0);
    }
  }
  
  /**
   * Sets input time.
   */
  public void setTime(double t) {
    setInputValue(t);
  }
}
