package physbeans.phys;

import physbeans.math.*;

/**
 * A single vertical plane between two regions with a different index of refraction.
 */
public class SinglePlane extends OpticalElement {
  
  protected double nl, nr;
  
  /** Creates a new instance of SinglePlane */
  public SinglePlane() {
    super();
    nl = 1.0;
    nr = 1.0;
  }
  
  
  /**
   * Gets the index of refraction of the medium to the left of the glass plane.
   */
  public double getIndexOfRefractionLeft() {
    return nl;
  }
  
  /**
   * Sets the index of refraction of the medium to the left of the glass plane.
   */
  public void setIndexOfRefractionLeft(double nl) {
    this.nl = nl;
    trigger();
  }
  
  /**
   * Gets the index of refraction of the medium to the right of the glass plane.
   */
  public double getIndexOfRefractionRight() {
    return nr;
  }
  
  /**
   * Sets the index of refraction of the medium to the right of the glass plane.
   */
  public void setIndexOfRefractionRight(double nr) {
    this.nr = nr;
    trigger();
  }
  
  /**
   * Adds additional point and changes the final angle of the ray if it
   * passes through the plane.<P>
   * If the maximal number of points is reached, it does nothing.
   */
  protected void extend(LightRay ray) {
    double x = ray.getLastPoint().get(0);
    if (ray.isRightGoing() && (x < pos)) {
      propagateThroughPlane(ray, pos, nl, nr);
    } else if (ray.isLeftGoing() && (x > pos)) {
      propagateThroughPlane(ray, pos, nr, nl);
    }
  }
  
  /**
   * Computes the propagation of a ray crossing a vertical plane at xs.
   * Indices of refraction are ni on the incoming side, no on the outgoing side.
   * Assumes that an intersection exists.
   */
  protected void propagateThroughPlane(LightRay ray, double xs, double ni, double no) {
    DVector p = ray.getLastPoint();
    double al = ray.getAngle();
    double y1 = p.get(1) + (xs - p.get(0))*Math.tan(al);
    ray.addPoint(xs, y1);
    
    double sal1 = ni/no * Math.sin(al);
    if (Math.abs(sal1) > 1) {            // total reflection
      ray.setAngle(Math.PI - al);
    } else {                             // refraction
      double al1 = Math.asin(sal1);
      al1 = (al > Math.PI/2)? Math.PI-al1 : al1;    // careful with asin!
      al1 = (al < -Math.PI/2)? -Math.PI-al1 : al1;    // careful with asin!
      ray.setAngle(al1);
    }
  }
}
