package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import physbeans.math.*;
import physbeans.views.*;
import java.io.*;

/**
 * View that displays a large horizontal arrow. It can be used to represent a
 * numerical value by the length or the width (or even the angle) of the arrow.
 */
public class ArrowView extends View implements Serializable {
  
  protected Color arrowColor;
  protected double arrowLength;
  protected double arrowWidth;
  protected double arrowHeadLength;
  protected double arrowHeadWidth;
  protected double x;
  protected double y;
  protected double alpha;
  
  protected Shape arrow;
  
  /** Creates a default ArrowView */
  public ArrowView() {
    super();
    arrowColor = Color.red;
    arrowLength = 1.0;
    arrowWidth = 0.3;
    arrowHeadLength = 0.3;
    arrowHeadWidth = 2.0;
    x = 0.0;
    y = 0.0;
    alpha = 0.0;
    //computeShape();
  }
  
  public void doLayout() {
    super.doLayout();
    // compute the arrow coordinates
    computeShape();
  }
  
  public void paintLayer(Graphics2D g2) {
    // draw horizontal arrow
    Color oldColor = g2.getColor();
    g2.setColor(arrowColor);
    g2.fill(arrow);
    g2.setColor(oldColor);
  }
  
  protected void computeShape() {
    float x1 = (float) ((1 - arrowHeadLength)*arrowLength);
    float x2 = (float) arrowLength;
    float y1 = (float) (0.5*arrowWidth);
    float y2 = (float) (arrowHeadWidth*y1);
    
    GeneralPath arrow1 = new GeneralPath();
    arrow1.moveTo(0.0f, y1);
    arrow1.lineTo(x1, y1);
    arrow1.lineTo(x1, y2);
    arrow1.lineTo(x2, 0.0f);
    arrow1.lineTo(x1, -y2);
    arrow1.lineTo(x1, -y1);
    arrow1.lineTo(0.0f, -y1);
    arrow1.closePath();
    
    AffineTransform rotator = AffineTransform.getTranslateInstance(x, y);
    rotator.rotate(alpha);
    arrow = rotator.createTransformedShape(arrow1);
    
    update();
  }
  
  /**
   * Gets length of the arrow in world coordinates.
   */
  public double getArrowLength() {
    return this.arrowLength;
  }
  
  /**
   * Sets length of the arrow in world coordinates.
   */
  public void setArrowLength(double arrowLength) {
    this.arrowLength = arrowLength;
    computeShape();
  }
  
  /**
   * Gets color of the arrow .
   */
  public Color getArrowColor() {
    return this.arrowColor;
  }
  
  /**
   * Sets color of the arrow .
   */
  public void setArrowColor(Color arrowColor) {
    this.arrowColor = arrowColor;
    update();
  }
  
  /**
   * Gets width of the arrow in world coordinates.
   */
  public double getArrowWidth() {
    return this.arrowWidth;
  }
  
  
  /**
   * Sets the width of the  arrow<br>
   */
  public void setArrowWidth(double val) {
    arrowWidth = val;
    computeShape();
  }
  
  /**
   * Gets length of the arrow head relative to the total arrow length.
   */
  public double getArrowHeadLength() {
    return this.arrowHeadLength;
  }
  
  /**
   * Sets length of the arrow head relative to the total arrow length.<br>
   * Should be in (0, 1).
   */
  public void setArrowHeadLength(double arrowHeadLength) {
    this.arrowHeadLength = arrowHeadLength;
    computeShape();
  }
  
  /**
   * Gets width of the arrow head relative to the total arrow width .
   */
  public double getArrowHeadWidth() {
    return this.arrowHeadWidth;
  }
  
  /**
   * Sets width of the arrow head relative to the total arrow width .
   * Should be > 1
   */
  public void setArrowHeadWidth(double arrowHeadWidth) {
    this.arrowHeadWidth = arrowHeadWidth;
    computeShape();
  }
  
  /**
   * Gets x coordinate of the base point.
   */
  public double getX() {
    return this.x;
  }
  
  /**
   * Sets x coordinate of the base point.
   */
  public void setX(double x) {
    this.x = x;
    computeShape();
  }
  
  /**
   * Gets y coordinate of the base point.
   */
  public double getY() {
    return this.y;
  }
  
  /**
   * Sets y coordinate of the base point.
   */
  public void setY(double y) {
    this.y = y;
    computeShape();
  }
  
  /**
   * Gets direction of arrow as angle relative to positive x direction.
   */
  public double getDirection() {
    return this.alpha;
  }
  
  /**
   * Sets direction of arrow as angle relative to positive x direction.
   */
  public void setDirection(double direction) {
    this.alpha = direction;
    computeShape();
  }
}
