package physbeans.views;

import physbeans.model.*;
import static java.lang.Math.*;
import static physbeans.math.Maths.sqr;

/**
 * View that draws a color image representing a GridWithBoundary that may have
 * complex values (i.e. it can be a ComplexGridWithBoundary).
 * The view shows either the real or imaginary part, the absolute value, its
 * square or the phase. 
 * The parameters maxVal and minVal refer to the real or imaginary case,
 * for all other cases they are internally adapted to cope with the change of
 * scale.
 */
public class ComplexGridView extends GridView {

  protected ComplexToRealEnum type;
  protected double[][] auxGrid;
  protected double minValReal, maxValReal; // store the values for the real case

  /** Creates a new instance of GridView */
  public ComplexGridView() {
    super();
    type = ComplexToRealEnum.REAL;
    minValReal = -1.0;
    maxValReal = 1.0;
  }

  /**
   * Gets the grid that will be displayed.
   * This can come directly from the model field (real or imaginary values)
   * or has to be computed here (absolute values or phases).
   * @Override
   */
  protected double[][] getGrid() {
    // assumes field != null (which is checked before)

    // model is a scalar field -> simply return its grid
    if (field instanceof ComplexGridWithBoundary) {
      ComplexGridWithBoundary cfield = (ComplexGridWithBoundary) field;
      switch (type) {
        case REAL:
          return cfield.getGridReal();
        case IMAG:
          return cfield.getGridImag();
        case ABS:
          createAbsGrid();
          return auxGrid;
        case ABS2:
          createAbs2Grid();
          return auxGrid;
        case PHASE:
          createPhaseGrid();
          return auxGrid;
        default:
          return cfield.getGridReal();
      }
    } else {
      return field.getGrid();
    }
  }

  protected void createAbsGrid() {
    // assumes field != null (which is checked before)

    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    if ((auxGrid == null) || (auxGrid[0].length != nx) || (auxGrid.length != ny)) {
      auxGrid = new double[ny][nx];
    }

    ComplexGridWithBoundary cfield = (ComplexGridWithBoundary) field;
    double[][] realGrid = cfield.getGridReal();
    double[][] imagGrid = cfield.getGridImag();

    for (int i = 0; i < nx; i++) {
      for (int j = 0; j < ny; j++) {
        auxGrid[j][i] = hypot(realGrid[j][i], imagGrid[j][i]);
      }
    }

  }

  protected void createAbs2Grid() {
    // assumes field != null (which is checked before)

    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    if ((auxGrid == null) || (auxGrid[0].length != nx) || (auxGrid.length != ny)) {
      auxGrid = new double[ny][nx];
    }

    ComplexGridWithBoundary cfield = (ComplexGridWithBoundary) field;
    double[][] realGrid = cfield.getGridReal();
    double[][] imagGrid = cfield.getGridImag();

    double x, y;
    for (int i = 0; i < nx; i++) {
      for (int j = 0; j < ny; j++) {
        x = realGrid[j][i];
        y = imagGrid[j][i];
        auxGrid[j][i] = x * x + y * y;
      }
    }

  }

  protected void createPhaseGrid() {
    // assumes field != null (which is checked before)

    int nx = field.getGridSizeX();
    int ny = field.getGridSizeY();
    if ((auxGrid == null) || (auxGrid[0].length != nx) || (auxGrid.length != ny)) {
      auxGrid = new double[ny][nx];
    }

    ComplexGridWithBoundary cfield = (ComplexGridWithBoundary) field;
    double[][] realGrid = cfield.getGridReal();
    double[][] imagGrid = cfield.getGridImag();

    for (int i = 0; i < nx; i++) {
      for (int j = 0; j < ny; j++) {
        auxGrid[j][i] = atan2(imagGrid[j][i], realGrid[j][i]);
      }
    }
  }

  /**
   * Gets the type of the complex view.
   * Possible values are
   * <ul>
   * <li>REAL: show real values</li>
   * <li>IMAG: show imaginary values</li>
   * <li>ABS: show absolute values</li>
   * <li>ABS2: show square of absolute values</li>
   * <li>PHASE: show phase values</li>
   * </ul>
   * Default is ABS.
   */
  public ComplexToRealEnum getType() {
    return type;
  }

  /**
   * Sets the type of the complex view.
   */
  public void setType(ComplexToRealEnum t) {
    type = t;

    switch (type) {
      case REAL:
      case IMAG:
        minVal = minValReal;
        maxVal = maxValReal;
        break;
      case ABS:
        minVal = 0.0;
        maxVal = max(abs(minValReal), abs(maxValReal));
        break;
      case ABS2:
        minVal = 0.0;
        maxVal = max(sqr(minValReal), sqr(maxValReal));
        break;
      case PHASE:
        minVal = -3.16; // slightly more than PI to cope with pixel size
        maxVal = 3.16;
        break;
      default:
        minVal = minValReal;
        maxVal = maxValReal;
    }
    update();
  }

  /**
   * Sets the type of the complex view using its ordinal number.
   */
  public void setTypeFromInt(int n) {
    setType(ComplexToRealEnum.values()[n]);
  }

  /**
   * Gets scalar value that is mapped to highest color (255).
   */
  public double getMaxVal() {
    return maxValReal;
  }

  /**
   * Sets scalar value that is mapped to highest color (255).<p>
   * Settings maxVal to Infinity leads to scaling with arctan
   */
  public void setMaxVal(double max) {
    maxVal = max;
    maxValReal = max;
    update();
  }

  /**
   * Gets scalar value that is mapped to lowest color (0).
   */
  public double getMinVal() {
    return minValReal;
  }

  /**
   * Sets scalar value that is mapped to lowest color (0).
   */
  public void setMinVal(double min) {
    minVal = min;
    minValReal = min;
    update();
  }
}
