package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import physbeans.model.*;
import physbeans.math.*;

/**
 * View that displays a DataBuffer as a number of graphs, each in its own color.
 * The first data column defines the common x values, the other lines represent
 * different corresponding y values.<br>
 * The plots can be line or point plots.
 */
public class DataBufferView extends View implements Serializable {
  
  protected DataBuffer model;
  protected boolean[] lineActive;
  protected boolean dotted, showMarker;
  protected int dotSize;
  protected float[] lineWidth;
  
  // color of the curves
  // if there are more lines than colors, this is used cyclically
  protected Color[] lineColor = {
    Color.green, Color.magenta, Color.cyan, Color.pink, Color.orange,
    Color.red, Color.blue, Color.yellow
  };
  
  
  /**
   * Creates a default DataBufferView.
   */
  public DataBufferView() {
    super();
    
    lineActive = new boolean[1];
    lineActive[0] = true;
    dotted = false;
    showMarker = false;
    dotSize = 1;
    lineWidth = new float[1];
  }
  
  /**
   * Paints the points of each Buffer as a polyline or a number of points.
   */
  public void paintLayer(Graphics2D g2) {
    if (model == null) {
      return;
    }
    
    int nPoints = model.getNData();
    DVector[] data = model.getData();
    double[] xPoints = data[0].getArray();
    
    for (int line = 1; line < data.length; line++) {
      if (!lineActive[line - 1]) {
        continue;
      }
      
      g2.setColor(lineColor[(line-1) % lineColor.length]);
      
      double[] yPoints = data[line].getArray();
      
      if (dotted) {
        for (int i = 0; i < nPoints; i++) {
          drawPoint(g2, xPoints[i], yPoints[i]);
        };
        
      } else {
        if (nPoints < 2) {
          return;
        }
        
        g2.setStroke(new BasicStroke(lineWidth[(line-1) % lineWidth.length]));
        drawPolyline(g2, xPoints, yPoints, nPoints);
        if (showMarker) {
          int pos = model.getNData() - 1;
          drawPoint(g2, xPoints[pos], yPoints[pos]);
        }
      }
    }
  }
  
  /**
   * Gets underlying model, i.e. the DataBuffer defining the points.
   */
  public DataBuffer getModel() {
    return model;
  }
  
  /**
   * Sets underlying model, i.e. the DataBuffer defining the points.
   */
  public void setModel(DataBuffer m) {
    model = m;
    
    int nLinesNew = model.getNBuffer() - 1;
    if (nLinesNew != lineActive.length) {
      lineActive = new boolean[nLinesNew];
      lineActive[0] = true;
    }
    
    update();
  }
  
  /**
   * gets drawing status of a line
   */
  public boolean getLineActive(int index) {
    return this.lineActive[index];
  }
  
  /**
   * gets drawing status of all lines
   */
  public boolean[] getLineActive() {
    return this.lineActive;
  }
  
  /**
   * sets drawing status of a line
   */
  public void setLineActive(int index, boolean lineActive) {
    this.lineActive[index] = lineActive;
    update();
  }
  
  /**
   * sets drawing status of all lines
   */
  public void setLineActive(boolean[] lineActive) {
    this.lineActive = lineActive;
    update();
  }
  
  /**
   * gets color of a line
   */
  public Color getLineColor(int index) {
    return this.lineColor[index];
  }
  
  /**
   * gets color of all lines
   */
  public Color[] getLineColor() {
    return this.lineColor;
  }
  
  /**
   * sets color of a line
   */
  public void setLineColor(int index, Color lineColor) {
    this.lineColor[index] = lineColor;
  }
  
  /**
   * sets color of all lines
   */
  public void setLineColor(Color[] lineColor) {
    this.lineColor = lineColor;
  }
  
  /**
   * Gets value of dotted flag.<br>
   * If true dots are drawn instead of lines.
   */
  public boolean isDotted() {
    return this.dotted;
  }
  
  /**
   * Sets value of dotted flag.<br>
   * If true dots are drawn instead of lines.
   */
  public void setDotted(boolean dotted) {
    this.dotted = dotted;
    update();
  }
  
  /**
   * Gets value of showMarker flag.<br>
   * If true, the latest data is displayed with a marker.
   */
  public boolean isShowMarker() {
    return showMarker;
  }
  
  /**
   * Sets value of showMarker flag.<br>
   * If true, the latest data is displayed with a marker.
   */
  public void setShowMarker(boolean showMarker) {
    this.showMarker = showMarker;
    update();
  }
  
  /**
   * Gets pixel size of dots drawn in dotted mode.
   */
  public int getDotSize() {
    return this.dotSize;
  }
  
  /**
   * Sets pixel size of dots drawn in dotted mode.
   */
  public void setDotSize(int dotSize) {
    this.dotSize = dotSize;
    update();
  }
  
  /**
   * Gets line widths of the lines to draw.
   */
  public float[] getLineWidths() {
    return lineWidth;
  }
  
  /**
   * Sets line widths of the lines to draw.<br>
   * If a value is 0, the line is drawn as thin as possible.
   */
  public void setLineWidths(float[] lw) {
    lineWidth = lw;
    update();
  }
  
  /**
   * Draws point at (x,y) as a filled rectangle of fixed pixel size.
   */
  protected void drawPoint(Graphics2D g2, double x, double y) {
    double rx = dotSize/2.0 * trafo.getScalingX();
    double ry = dotSize/2.0 * trafo.getScalingY();
    
    // circles of radius 1 sometimes disappear ("smaller than a pixel")
    // in that case I use rectangles
    
    if (dotSize == 1) {
      Rectangle2D rect = new Rectangle2D.Double(x - rx, y - ry, 2*rx, 2*ry);
      g2.fill(rect);
    } else {
      Ellipse2D circ = new Ellipse2D.Double(x - rx, y - ry, 2*rx, 2*ry);
      g2.fill(circ);
    }
  }
  
  /**
   * Draws first np elements of xv, yv as a polyline.
   */
  protected void drawPolyline(Graphics2D g2, double[] xv, double[] yv, int np) {
    GeneralPath polyline = new GeneralPath();
    polyline.moveTo((float) xv[0], (float) yv[0]);
    
    for (int i = 1; i < np; i++) {
      polyline.lineTo((float) xv[i], (float) yv[i]);
    };
    
    g2.draw(polyline);
  }
  
}
