package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import static java.lang.Math.*;
import physbeans.math.*;

/**
 * Displays a beam sitting on two counterrotating wheels.
 */
public class DrivenBeamView extends RestartableView implements Serializable {

  protected double b, h, r, l;

  /**
   * Creates a standard DrivenBeamView.
   */
  public DrivenBeamView() {
    super();

    model = new DVector(0.5, 0.5);
    initialValues = new DVector(0.0, 0.0);

    b = 1.0;
    h = 0.2;
    r = 0.2;
    l = 0.5;
  }

  /**
   * Redraws the complete pendulum according to driving parameters.
   */
  public void paintLayer(Graphics2D g) {

    double x = model.get(0);
    double phi = model.get(1);

    drawWheel(g, -l / 2, phi);
    drawWheel(g, l / 2, -phi);
    drawBar(g, x);
  }

  /**
   * Sets displacement of beam.
   * Conveniance function to simplify routing.
   */
  public void setBeamDisplacement(double x) {
    model.set(0, x);
    update();
  }

  /**
   * Sets angle of wheels.
   * Conveniance function to simplify routing.
   */
  public void setWheelAngle(double phi) {
    model.set(1, phi);
    update();
  }

  /**
   * Gets beam length.
   */
  public double getBeamLength() {
    return b;
  }

  /**
   * Sets beam length.
   */
  public void setBeamLength(double b) {
    this.b = b;
    update();
  }

  /**
   * Gets beam height.
   */
  public double getBeamHeight() {
    return h;
  }

  /**
   * Sets beam height.
   */
  public void setBeamHeight(double h) {
    this.h = h;
    update();
  }

  /**
   * Gets wheel radius.
   */
  public double getWheelRadius() {
    return r;
  }

  /**
   * Sets wheel radius.
   */
  public void setWheelRadius(double r) {
    this.r = r;
    update();
  }

  /**
   * Gets wheel distance.
   */
  public double getWheelDistance() {
    return l;
  }

  /**
   * Sets wheel distance.
   */
  public void setWheelDistance(double l) {
    this.l = l;
    update();
  }

  /**
   * Draws a wheel a x position xPos, rotated by phi.
   */
  void drawWheel(Graphics2D g, double xPos, double phi) {
    Ellipse2D wheel = new Ellipse2D.Double();
    wheel.setFrameFromCenter(xPos, 0.0, xPos - r, r);

    double c = r*cos(phi);
    double s = r*sin(phi);
    Line2D l1 = new Line2D.Double(s + xPos,  c, -s + xPos, -c);
    Line2D l2 = new Line2D.Double(c + xPos, -s, -c + xPos,  s);

    g.draw(wheel);
    g.draw(l1);
    g.draw(l2);
  }

  /**
   * Draws the beam, shifted by x.
   */
  void drawBar(Graphics2D g, double x) {
    Rectangle2D beam = new Rectangle2D.Double(x - b / 2, r, b, h);
    g.draw(beam);
  }
}
