package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import physbeans.math.*;
import static java.lang.Math.*;

/**
 * Displays a spring with given two-dimensional end positions.
 */
public class SingleSpringView extends View implements Serializable {
  
  protected int nTurns;
  protected double springWidth;
  protected double lBegin, lEnd;
  protected boolean coiled;
  protected DVector leftPosition;
  protected DVector rightPosition;
  
  /**
   * create a standard horizontal SpringView
   */
  public SingleSpringView() {
    super();
    
    nTurns = 5;
    springWidth = 0.8;
    lBegin = 0.1;
    lEnd   = 0.1;
    
    leftPosition = new DVector(0.0, 0.0);
    rightPosition = new DVector(1.0, 0.0);
    coiled = true;
  }
  
  /**
   * Draws the spring.
   * <P>Springs with a negative width are not drawn at all.
   */
  public void paintLayer(Graphics2D g) {
    if (springWidth < 0) {
      return;
    }
    
    if (coiled) {
      drawCoiledSpring(g);
    } else {
      drawSimpleSpring(g);
    }
  }
  
  /**
   * Gets number of turns of the spring
   */
  public int getNTurns() {
    return this.nTurns;
  }
  
  /**
   * Sets number of turns of the spring
   */
  public void setNTurns(int nTurns) {
    this.nTurns = nTurns;
    update();
  }
  
  /**
   * Gets width of the spring
   */
  public double getSpringWidth() {
    return this.springWidth;
  }
  
  /**
   * Sets width of the spring
   */
  public void setSpringWidth(double springWidth) {
    this.springWidth = springWidth;
    update();
  }
  
  /**
   * Gets length of the straight line at the beginning of the spring
   */
  public double getStartLineLength() {
    return this.lBegin;
  }
  
  /**
   * Sets length of the straight line at the beginning of the spring
   */
  public void setStartLineLength(double lBegin) {
    this.lBegin = lBegin;
    update();
  }
  
  /**
   * Gets length of the straight line at the end of the spring
   */
  public double getEndLineLength() {
    return this.lEnd;
  }
  
  /**
   * Sets length of the straight line at the end of the spring
   */
  public void setEndLineLength(double lEnd) {
    this.lEnd = lEnd;
    update();
  }
  
  /**
   * Gets position of the left end point of the spring.
   */
  public DVector getLeftPosition() {
    return leftPosition;
  }
  
  /**
   * Sets position of the left end point of the spring.
   */
  public void setLeftPosition(DVector position) {
    leftPosition = position;
  }
  
  /**
   * Gets position of the right end point of the spring.
   */
  public DVector getRightPosition() {
    return rightPosition;
  }
  
  /**
   * Sets position of the right end point of the spring.
   */
  public void setRightPosition(DVector position) {
    rightPosition = position;
  }
  
  /**
   * Gets coiled flag.
   *<P>If true, a symbolic coil is drawn, else a simple rubber band.
   */
  public boolean isCoiled() {
    return coiled;
  }
  
  /**
   * Sets coiled flag.
   *<P>If true, a symbolic coil is drawn, else a simple rubber band.
   */
  public void setCoiled(boolean b) {
    coiled = b;
  }
  
  /**
   * draws a horizontal spring between pixel positions left and right,
   * centered vertically.
   */
  protected void drawHorizontalSpring(Graphics2D g, double left, double right) {
    double x = 0;
    
    GeneralPath spring = new GeneralPath();
    
    // draw line at the beginning of the spring
    spring.moveTo((float)left,(float)x);
    spring.lineTo((float)(left + lBegin), (float)x);
    
    // draw spiral
    double length = right - left - lBegin - lEnd;
    double start, end, mid1, mid2;
    for (int i=0; i<nTurns; i++) {
      start = left + lBegin + i * length/(nTurns);
      end = left + lBegin + (i+1) * length/(nTurns);
      mid1 = (3*start + end)/4.0;
      mid2 = (start + 3*end)/4.0;
      spring.lineTo((float)mid1, (float)(x + springWidth/2));
      spring.lineTo((float)mid2, (float)(x - springWidth/2));
      spring.lineTo((float)end, (float)x);
    }
    // draw line at the end of the spring
    spring.lineTo((float)right, (float)x);
    
    g.draw(spring);
  }
  
  /**
   * draws a vertical spring between pixel positions up and down,
   * centered horizontally.
   */
  protected void drawVerticalSpring(Graphics2D g, double up, double down) {
    double y = 0;
    
    GeneralPath spring = new GeneralPath();
    
    // draw line at the beginning of the spring
    spring.moveTo((float)y,(float)up);
    spring.lineTo((float)y, (float)(up - lBegin));
    
    // draw spiral
    double length = up - down - lBegin - lEnd;
    double start, end, mid1, mid2;
    for (int i=0; i<nTurns; i++) {
      start = up - lBegin - i * length/(nTurns);
      end = up - lBegin - (i+1) * length/(nTurns);
      mid1 = (3*start + end)/4.0;
      mid2 = (start + 3*end)/4.0;
      spring.lineTo((float)(y - springWidth/2), (float)mid1);
      spring.lineTo((float)(y + springWidth/2), (float)mid2);
      spring.lineTo((float)y, (float)end);
    }
    // draw line at the end of the spring
    spring.lineTo((float)y, (float)down);
    
    g.draw(spring);
  }
  
  /**
   * draws a rubber spring
   */
  protected void drawSimpleSpring(Graphics2D g) {
    Color oldColor = g.getColor();
    g.setColor(Color.PINK);
    
    GeneralPath spring = new GeneralPath();
    spring.moveTo((float) leftPosition.x(),(float) leftPosition.y());
    spring.lineTo((float) rightPosition.x(), (float) rightPosition.y());
    g.draw(spring);
    
    g.setColor(oldColor);
  }
  
  /**
   * draws a coil spring
   */
  protected void drawCoiledSpring(Graphics2D g) {
    float lx = (float) leftPosition.x();
    float ly = (float) leftPosition.y();
    float rx = (float) rightPosition.x();
    float ry = (float) rightPosition.y();
    
    float lTotal = (float) hypot(rx - lx, ry - ly);
    float lBeginF = (float) lBegin;
    float lEndF = (float) lEnd;
    GeneralPath spring = new GeneralPath();
    
    // first draw it horizontally
    
    // draw line at the beginning of the spring
    spring.moveTo(lx,ly);
    spring.lineTo(lx + (float) lBegin, ly);
    
    // draw spiral
    float length = lTotal - lBeginF - lEndF;
    float start, end, mid1, mid2;
    for (int i=0; i<nTurns; i++) {
      start = lx + lBeginF + i * length/(nTurns);
      end = lx + lBeginF + (i+1) * length/(nTurns);
      mid1 = (3*start + end)/4.0f;
      mid2 = (start + 3*end)/4.0f;
      spring.lineTo(mid1, ly + (float) springWidth/2);
      spring.lineTo(mid2, ly - (float) springWidth/2);
      spring.lineTo(end, ly);
    }
    // draw line at the end of the spring
    spring.lineTo(lx + lTotal, ly);
    
    // then rotate it
    
    double theta = atan2(ry - ly, rx - lx);
    AffineTransform rot = AffineTransform.getRotateInstance(theta, lx, ly);
    
    g.draw(spring.createTransformedShape(rot));
  }
}
