package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;

import physbeans.inout.*;
import physbeans.phys.*;
import physbeans.math.*;
import physbeans.model.*;

import static java.lang.Math.*;     // readable math at last!
import static physbeans.math.Maths.*;

/**
 * Displays a spherical mirror.<p>
 * Additionally the active plane and the focal point can be shown.
 * Optionally the radius used for drawing can be different from the actual
 * radius of the mirror. This is useful if the SphericalMirror uses
 * the paraxial approximation.
 */
public class SphericalMirrorView extends View {
  
  protected SphericalMirror model;
  protected Color mirrorColor, principalPlaneColor;
  protected double drawingRadius;
  protected boolean showPrincipalPlane;
  protected boolean showFocalPoint;
  protected PointView focalView;
  
  /** Creates a new instance of SphericalMirrorView */
  public SphericalMirrorView() {
    mirrorColor = Color.BLACK;
    principalPlaneColor = new Color(128, 128, 255);
    showPrincipalPlane = true;
    showFocalPoint = false;
    drawingRadius = Double.NaN;
    focalView = new PointView();
    focalView.setPointRadius(0.2);
    focalView.setPointColor(Color.black);
  }
  
  public void paintLayer(Graphics2D g2) {
    if (model == null) {
      return;  // do nothing
    }
    
    double x = model.getPosition();
    double h = model.getHeight();
    double r = drawingRadius*sign(model.getRadius());
    if (Double.isNaN(r)) {
      r = model.getRadius();
    }
    
    double al = 180/PI * asin(h/r);
    Line2D line = new Line2D.Double(x, h, x, -h);
    Arc2D arc;
    if (r > 0) {
      arc = new Arc2D.Double(x, -r, 2*r, 2*r, 180-al, 2*al, Arc2D.OPEN);
    } else {
      arc = new Arc2D.Double(x+2*r, r, -2*r, -2*r, -al, 2*al, Arc2D.OPEN);
    }
    
    if (showPrincipalPlane) {
      g2.setColor(principalPlaneColor);
      g2.draw(line);
    }
    
    g2.setColor(mirrorColor);
    g2.draw(arc);
    
    if (showFocalPoint) {
      focalView.setModel(model.getFocalPoints());
      focalView.paintLayer(g2);
    }
  }
  
  /**
   * Gets underlying model defining the mirror to paint.
   */
  public SphericalMirror getModel() {
    return model;
  }
  
  /**
   * Sets underlying model defining the mirror to paint.
   */
  public void setModel(SphericalMirror model) {
    this.model = model;
    update();
  }
  
  /**
   * Gets the line color of the mirror.
   */
  public Color getMirrorColor() {
    return this.mirrorColor;
  }
  
  /**
   * Sets the line color of the mirror.
   */
  public void setMirrorColor(Color mirrorColor) {
    this.mirrorColor = mirrorColor;
    update();
  }
  
  /**
   * Gets the line color of the active plane.
   */
  public Color getPrincipalPlaneColor() {
    return this.principalPlaneColor;
  }
  
  /**
   * Sets the line color of the active plane.
   */
  public void setPrincipalPlaneColor(Color principalPlaneColor) {
    this.principalPlaneColor = principalPlaneColor;
    update();
  }
  
  /**
   * Gets the value of the showFocalPoint flag.
   */
  public boolean isShowFocalPoint() {
    return this.showFocalPoint;
  }
  
  /**
   * Sets the value of the showFocalPoint flag.
   */
  public void setShowFocalPoint(boolean showFocalPoint) {
    this.showFocalPoint = showFocalPoint;
    update();
  }
  
  /**
   * Gets the value of the showPrincipalPlane flag.
   */
  public boolean isShowPrincipalPlane() {
    return this.showPrincipalPlane;
  }
  
  /**
   * Sets the value of the showPrincipalPlane flag.
   */
  public void setShowPrincipalPlane(boolean showPrincipalPlane) {
    this.showPrincipalPlane = showPrincipalPlane;
    update();
  }
  
  /**
   * Gets the mirror radius used for the display.
   * If it is NaN the radius of the mirror is used. <p>
   * For a mirror in paraxial approximation showing a large curvature can be
   * disturbing due to the large differences in the model and the drawing.
   */
  public double getDrawingRadius() {
    return this.drawingRadius;
  }
  
  /**
   * Sets the mirror radius used for the display.
   * If it is NaN the radius of the mirror is used. <p>
   */
  public void setDrawingRadius(double drawingRadius) {
    this.drawingRadius = drawingRadius;
    update();
  }
  
  /**
   * Gets radius of the circle representing the focal point.
   */
  public double getFocalPointRadius() {
    return focalView.getPointRadius();
  }
  
  /**
   * Sets radius of the circle representing the focal point.
   */
  public void setFocalPointRadius(double r) {
    focalView.setPointRadius(r);
    update();
  }
  
  /**
   * Sets trafo between screen and world coordinates.
   */
  public void setTrafo(ScreenWorldTrafo trafo) {
    super.setTrafo(trafo);
    focalView.setTrafo(trafo);
  }
  
}
