package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import static java.lang.Math.*;

/**
 * Displays a ThreeColorMixer as three overlapping discs.
 */
public class ThreeColorMixerView extends View {
  
  protected Color[] cols;
  protected double r;
  protected static final double TRIFAC = 1.0/sqrt(3.0);
  
  protected Area[] areas;
  
  /** Creates a new instance of ThreeColorMixerView */
  public ThreeColorMixerView() {
    r = 1.0;
    cols = new Color[] {
      Color.BLUE, Color.BLUE, Color.BLUE, Color.ORANGE,
      Color.ORANGE, Color.ORANGE, Color.BLUE
    };
    createShapes();
  }
  
  /**
   * Gets the radius of the circles.
   */
  public double getRadius() {
    return r;
  }
  
  /**
   * Sets the radius of the circles.
   */
  public void setRadius(double r1) {
    r = r1;
    createShapes();
    update();
  }
  
  /**
   * Define the colors to draw in the circle segments.
   * Order: c1, c2, c3, c12, c23, c31, c123.
   */
  public void setModel(Color[] newCols) {
    cols = newCols;
    update();
  }
  
  public void paintLayer(Graphics2D g2) {
    for (int i = 0; i < 7; i++) {
      g2.setColor(cols[i]);
      g2.fill(areas[i]);
    }
  }
  
  /**
   * Create all geometric objects generated by intersection of two circles.
   */
  protected void createShapes() {
    Point2D p1 = new Point2D.Double(-0.5, -0.5*TRIFAC);
    Point2D p2 = new Point2D.Double( 0.5, -0.5*TRIFAC);
    Point2D p3 = new Point2D.Double( 0.0,      TRIFAC);
    
    Ellipse2D.Double c1 = new Ellipse2D.Double();
    Ellipse2D.Double c2 = new Ellipse2D.Double();
    Ellipse2D.Double c3 = new Ellipse2D.Double();
    c1.setFrameFromCenter(p1.getX(), p1.getY(), p1.getX() + r, p1.getY() + r);
    c2.setFrameFromCenter(p2.getX(), p2.getY(), p2.getX() + r, p2.getY() + r);
    c3.setFrameFromCenter(p3.getX(), p3.getY(), p3.getX() + r, p3.getY() + r);
    
    Area a1, a2, a3, a12, a23, a31, a123;
    a1 = new Area(c1);
    a2 = new Area(c2);
    a3 = new Area(c3);
    a12 = new Area(c1);
    a23 = new Area(c2);
    a31 = new Area(c3);
    a123 = new Area(c3);
    
    a12.intersect(a2);
    a23.intersect(a3);
    a31.intersect(a1);
    
    a123.intersect(a12);
    
    a1.subtract(a12);
    a1.subtract(a31);
    a2.subtract(a12);
    a2.subtract(a23);
    a3.subtract(a23);
    a3.subtract(a31);
    
    a12.subtract(a123);
    a23.subtract(a123);
    a31.subtract(a123);
    
    areas = new Area[] {a1, a2, a3, a12, a23, a31, a123};
  }
}
