package physbeans.views;

import java.awt.*;
import java.io.*;
import physbeans.inout.*;
import physbeans.math.*;

/**
 * Displays two masses represented by rectangles, and three connecting springs.
 * Zero positions are at 1/3 and 2/3 of panel. Amplitude gives scale factor
 * from par to pixel.
 */
public class TwoMassSpringView extends RestartableView implements Serializable {
  
  protected double offset;
  
  protected SingleSpringView spring1;
  protected SingleSpringView spring2;
  protected SingleSpringView spring3;
  protected SingleMassView mass1;
  protected SingleMassView mass2;
  
  protected DVector m1Pos;
  protected DVector m2Pos;
  
  
  public TwoMassSpringView() {
    super();
    
    initialValues = new DVector(1.0, 0.0);
    model = new DVector(1.0, 0.0);
    offset = 1.4;
    
    spring1 = new SingleSpringView();
    spring2 = new SingleSpringView();
    spring3 = new SingleSpringView();
    mass1 = new SingleMassView();
    mass2 = new SingleMassView();
    
    double extralong = spring1.getEndLineLength() + mass1.getMassWidth()/2;
    // equal for both masses and all three springs
    
    spring1.setEndLineLength(extralong);
    spring2.setStartLineLength(extralong);
    spring2.setEndLineLength(extralong);
    spring3.setStartLineLength(extralong);
    
    spring1.setLeftPosition(new DVector(-4.2, 0.0));
    spring3.setRightPosition(new DVector(4.2, 0.0));
    
    m1Pos = new DVector(2);
    m2Pos = new DVector(2);
  }
  
  /**
   * redraw the complete spring according to driving parameter
   */
  public void paintLayer(Graphics2D g) {
    DVector x1 = new DVector(model.get(0) - offset, 0.0);
    DVector x2 = new DVector(model.get(1) + offset, 0.0);
    
    spring1.setRightPosition(x1);
    spring2.setLeftPosition(x1);
    spring2.setRightPosition(x2);
    spring3.setLeftPosition(x2);
    
    spring1.paintLayer(g);
    spring2.paintLayer(g);
    spring3.paintLayer(g);
        
    mass1.setPosition(x1);
    mass2.setPosition(x2);
    mass1.paintLayer(g);
    mass2.paintLayer(g);
  }
  
  /**
   * Gets offset of the coordinates.<br>
   * This is the x position of mass 1 at equilibrium, the equilibrium position
   *  of mass 2 is -offset.
   */
  public double getOffset() {
    return this.offset;
  }
  
  /**
   * Sets offset of the coordinates.<br>
   * This is the x position of mass 1 at equilibrium, the equilibrium position
   *  of mass 2 is -offset.
   */
  public void setOffset(double offset) {
    this.offset = offset;
    update();
  }
  
  /**
   * used by LayeredScreen
   */
  public void setTrafo(ScreenWorldTrafo trafo) {
    super.setTrafo(trafo);
    spring1.setTrafo(trafo);
    spring2.setTrafo(trafo);
    spring3.setTrafo(trafo);
    mass1.setTrafo(trafo);
    mass2.setTrafo(trafo);
  }
}
